/* Scrutari - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets;

import fr.exemole.sctserver.SctContext;
import fr.exemole.sctserver.SctContextConstants;
import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.directory.SctServerDirectory;
import fr.exemole.sctserver.servlets.selection.SelectionManager;
import fr.exemole.sctserver.storage.SctServerStorage;
import net.mapeadores.util.localisation.ResourceItem;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.XMLUtils;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import javax.servlet.ServletContext;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariContext implements ConfigErrorKey {

    private final static String SCRUTARICONTEXT_KEY = "fr.exemole.ScrutariContext";
    private String uuidString = "";
    private String confDirString = "";
    private String varDirString = "";
    private File confDir = null;
    private File varDir = null;
    private Map<String, InternalSctContext> sctContextMap = new TreeMap<String, InternalSctContext>();
    private Map<String, Verrou> nameMap = new HashMap<String, Verrou>();
    private ResourceItem errorKey;

    private ScrutariContext() {
    }

    public boolean hasError() {
        return (errorKey != null);
    }

    public ResourceItem getErrorKey() {
        return errorKey;
    }

    public SctContext getSctContext(String name) {
        InternalSctContext sctContext = sctContextMap.get(name);
        if (sctContext == null) {
            sctContext = createSctContext(name);
        }
        return sctContext;
    }

    public List<String> getNameList() {
        ArrayList<String> list = new ArrayList<String>();
        list.addAll(nameMap.keySet());
        return list;
    }

    private InternalSctContext createSctContext(String name) {
        Verrou verrou = nameMap.get(name);
        if (verrou == null) {
            return null;
        }
        InternalSctContext sctContext = null;
        synchronized (verrou) {
            sctContext = sctContextMap.get(name);
            if (sctContext == null) {
                sctContext = buildSctContext(name);
                sctContextMap.put(name, sctContext);
            }
        }
        return sctContext;
    }

    public static ScrutariContext getScrutariContext(ServletContext servletContext) {
        ScrutariContext scrutariContext = (ScrutariContext) servletContext.getAttribute(SCRUTARICONTEXT_KEY);
        if (scrutariContext == null) {
            scrutariContext = createScrutariContext(servletContext);
        }
        return scrutariContext;
    }

    private static synchronized ScrutariContext createScrutariContext(ServletContext servletContext) {
        ScrutariContext scrutariContext = (ScrutariContext) servletContext.getAttribute(SCRUTARICONTEXT_KEY);
        if (scrutariContext == null) {
            scrutariContext = new ScrutariContext();
            scrutariContext.reload(servletContext);
            servletContext.setAttribute(SCRUTARICONTEXT_KEY, scrutariContext);
        }
        return scrutariContext;
    }

    public synchronized boolean reload(ServletContext servletContext) {
        errorKey = null;
        String scrutariConfFileString = servletContext.getInitParameter("scrutariConfFile");
        if (scrutariConfFileString != null) {
            scrutariConfFileString = scrutariConfFileString.trim();
            if (scrutariConfFileString.length() == 0) {
                scrutariConfFileString = null;
            }
        }
        if (scrutariConfFileString == null) {
            errorKey = new ResourceItem(INIT_PARAMETER);
            return false;
        }
        File scrutariConfFile = new File(scrutariConfFileString);
        if (!scrutariConfFile.exists()) {
            errorKey = new ResourceItem(MISSING_CONF_FILE, scrutariConfFileString);
            return false;
        }
        if (scrutariConfFile.isDirectory()) {
            errorKey = new ResourceItem(CONF_FILE_DIRECTORY, scrutariConfFileString);
            return false;
        }
        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document doc = db.parse(scrutariConfFile);
            return readConf(doc.getDocumentElement());
        } catch (ParserConfigurationException pce) {
            pce.printStackTrace();
            errorKey = new ResourceItem(PARSER_CONFIG, pce.getMessage());
            return false;
        } catch (IOException ioe) {
            String[] val = new String[2];
            val[0] = scrutariConfFileString;
            val[1] = ioe.getMessage();
            errorKey = new ResourceItem(CONF_FILE_IOERROR, val);
            return false;
        } catch (SAXException sax) {
            String[] val = new String[2];
            val[0] = scrutariConfFileString;
            val[1] = sax.getMessage();
            errorKey = new ResourceItem(XML_PARSE, val);
            return false;
        }
    }

    private boolean readConf(Element element_xml) {
        String nvUuidString = null;
        String nvConfDirString = null;
        String nvVarDirString = null;
        Map<String, Verrou> hereMap = new HashMap<String, Verrou>();
        NodeList nodeList = element_xml.getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++) {
            Node nd = nodeList.item(i);
            if (nd instanceof Element) {
                Element el = (Element) nd;
                String tagName = el.getTagName();
                if (tagName.equals("sctserver")) {
                    String name = el.getAttribute("name").trim();
                    if (StringUtils.isTechnicalName(name, true)) {
                        hereMap.put(name, new Verrou());
                    }
                } else if (tagName.equals("conf-dir")) {
                    nvConfDirString = XMLUtils.getData(el);
                } else if (tagName.equals("var-dir")) {
                    nvVarDirString = XMLUtils.getData(el);
                } else if (tagName.equals("uuid")) {
                    nvUuidString = XMLUtils.getData(el);
                }
            }
        }
        boolean globalChange = false;
        if ((nvUuidString == null) || (nvUuidString.length() == 0)) {
            errorKey = new ResourceItem(XML_UUID_MISSING);
            return false;
        }
        if ((nvConfDirString == null) || (nvConfDirString.length() == 0)) {
            errorKey = new ResourceItem(XML_CONFDIR_MISSING);
            return false;
        }
        if ((nvVarDirString == null) || (nvVarDirString.length() == 0)) {
            errorKey = new ResourceItem(XML_VARDIR_MISSING);
            return false;
        }
        if (!nvUuidString.equals(uuidString)) {
            uuidString = nvUuidString;
            globalChange = true;
        }
        if (!nvConfDirString.equals(confDirString)) {
            File dir = testDir(nvConfDirString, "<conf-dir>");
            if (dir != null) {
                confDirString = nvConfDirString;
                confDir = dir;
                globalChange = true;
            } else {
                return false;
            }
        }
        if (!nvVarDirString.equals(varDirString)) {
            File dir = testDir(nvVarDirString, "<var-dir>");
            if (dir != null) {
                varDirString = nvVarDirString;
                varDir = dir;
                globalChange = true;
            } else {
                return false;
            }
        }
        nameMap.clear();
        nameMap.putAll(hereMap);
        if (globalChange) {
            sctContextMap.clear();
        } else {
            Map<String, InternalSctContext> newMap = new HashMap<String, InternalSctContext>();
            for (String name : nameMap.keySet()) {
                InternalSctContext sctContext = sctContextMap.get(name);
                if (sctContext != null) {
                    newMap.put(name, sctContext);
                }
            }
            sctContextMap = newMap;
        }
        return true;
    }

    private File testDir(String path, String elTag) {
        String[] erroval = new String[2];
        erroval[0] = elTag;
        erroval[1] = path;
        File dir = new File(path);
        if (!dir.exists()) {
            errorKey = new ResourceItem(XML_PATH_NOTEXIST, erroval);
            return null;
        }
        if (!dir.isDirectory()) {
            errorKey = new ResourceItem(XML_PATH_NOTDIR, erroval);
            return null;
        }
        return dir;
    }

    private InternalSctContext buildSctContext(String name) {
        File sctserverConfDir = new File(confDir, name);
        if (!sctserverConfDir.exists()) {
            sctserverConfDir.mkdir();
        }
        File sctserverVarDir = new File(varDir, name);
        if (!sctserverVarDir.exists()) {
            sctserverVarDir.mkdir();
        }
        File selectionManagerDir = new File(sctserverConfDir, "selection");
        SctServerStorage sctServerStorage = new SctServerDirectory(sctserverConfDir, sctserverVarDir);
        SctServer sctServer = new SctServer(name, sctServerStorage);
        SelectionManager selectionManager = new SelectionManager(selectionManagerDir);
        InternalSctContext sctContext = new InternalSctContext(sctServer);
        sctContext.putContextObject(SctContextConstants.SELECTIONMANAGER_NAME, selectionManager);
        sctContext.putContextObject(SctContextConstants.UUID_NAME, uuidString);
        return sctContext;
    }


    private static class Verrou {
    }


    private static class InternalSctContext implements SctContext {

        private SctServer SctServer;
        private Map<String, Object> contextObjectMap = new HashMap<String, Object>();

        InternalSctContext(SctServer SctServer) {
            this.SctServer = SctServer;
        }

        public SctServer getSctServer() {
            return SctServer;
        }

        public Object getContextObject(String objectName) {
            return contextObjectMap.get(objectName);
        }

        private void putContextObject(String objectName, Object object) {
            contextObjectMap.put(objectName, object);
        }

    }

}
