/* Scrutari - Copyright (c) 2011-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.feed;

import fr.exemole.sctserver.SctContext;
import fr.exemole.sctserver.SctContextConstants;
import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.feed.SctFeed;
import fr.exemole.sctserver.feed.SctFeedEntry;
import fr.exemole.sctserver.feed.SctFeedMetadata;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Date;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.scrutari.data.BaseData;
import net.mapeadores.scrutari.data.CorpusData;
import net.mapeadores.scrutari.data.FicheData;
import net.mapeadores.scrutari.data.MotcleData;
import net.mapeadores.scrutari.data.ThesaurusData;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.ScrutariDBName;
import net.mapeadores.scrutari.db.util.LibelleUtils;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleHolder;
import net.mapeadores.util.xml.XMLPart;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;
import net.scrutari.datauri.BaseURI;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.datauri.FicheURI;
import net.scrutari.datauri.MotcleURI;
import net.scrutari.datauri.ThesaurusURI;
import net.scrutari.datauri.tree.BaseNode;
import net.scrutari.datauri.tree.CorpusNode;
import net.scrutari.datauri.tree.FicheNode;
import net.scrutari.datauri.tree.MotcleNode;
import net.scrutari.datauri.tree.ThesaurusNode;
import net.scrutari.datauri.tree.URITree;


/**
 *
 * @author Vincent Calame
 */
public class UriTreeAtom implements ResponseHandler {

    private SctServer sctServer;
    private SctContext sctContext;
    private LocalisationInfo localisationInfo;
    private SctFeed sctFeed;
    private FeedOptions feedOptions;

    public UriTreeAtom(SctContext sctContext, LocalisationInfo localisationInfo, FeedOptions feedOptions) {
        this.sctContext = sctContext;
        this.sctServer = sctContext.getSctServer();
        this.localisationInfo = localisationInfo;
        this.feedOptions = feedOptions;
        sctFeed = sctServer.getFeed();
    }

    public long getLastModified() {
        return sctFeed.getLastModified();
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        DataAccess dataAccess = sctServer.getScrutariDB().openDataAccess();
        response.setContentType("application/atom+xml;charset=UTF-8");
        PrintWriter pw = response.getWriter();
        XMLWriter xmlWriter = XMLUtils.getXMLWriter(pw);
        xmlWriter.appendXMLDeclaration();
        FeedXMLPart feedXMLPart = new FeedXMLPart(xmlWriter, dataAccess);
        feedXMLPart.appendFeed();
        pw.close();
        dataAccess.close();
    }


    private class FeedXMLPart extends XMLPart {

        private String feedId;
        private DataAccess dataAccess;

        public FeedXMLPart(XMLWriter xmlWriter, DataAccess dataAccess) {
            super(xmlWriter);
            this.feedId = getFeedId();
            this.dataAccess = dataAccess;
        }

        public void appendFeed() throws IOException {
            startOpenTag("feed");
            addNameSpaceAttribute(null, "http://www.w3.org/2005/Atom");
            endOpenTag();
            addSimpleElement("title", getTitle());
            addSimpleElement("updated", sctFeed.getFeedUpdated());
            openTag("author");
            addSimpleElement("name", getAuthorName());
            closeTag("author");
            addSimpleElement("id", feedId);
            int entryCount = sctFeed.getSctFeedEntryCount();
            for (int i = 0; i < entryCount; i++) {
                SctFeedEntry entry = sctFeed.getSctFeedEntry(i);
                appendFeedEntry(entry);
            }
            closeTag("feed");
        }

        private String getTitle() {
            String title = null;
            SctFeedMetadata metadata = sctFeed.getMetadata();
            if (metadata != null) {
                LibelleHolder titleLib = metadata.getAuthorName();
                if (titleLib != null) {
                    title = titleLib.getLibelleString(localisationInfo.getWorkingLangInteger(), false);
                }
            }
            if ((title == null) || (title.length() == 0)) {
                title = "Scrutari " + sctServer.getName();
            }
            return title;
        }

        private String getAuthorName() {
            String authorName = null;
            SctFeedMetadata metadata = sctFeed.getMetadata();
            if (metadata != null) {
                LibelleHolder authorNameLib = metadata.getAuthorName();
                if (authorNameLib != null) {
                    authorName = authorNameLib.getLibelleString(localisationInfo.getWorkingLangInteger(), false);
                }
            }
            if ((authorName == null) || (authorName.length() == 0)) {
                authorName = sctServer.getName();
            }
            return authorName;
        }

        private void appendFeedEntry(SctFeedEntry entry) throws IOException {
            Date date = entry.getDate();
            URITree uriTree = sctServer.getSctServerStorage().getURITree(date);
            if (uriTree == null) {
                return;
            }
            int workingLangInteger = localisationInfo.getWorkingLangInteger();
            ContentBuffer contentBuffer = new ContentBuffer(dataAccess, workingLangInteger, getCurrentIndentValue() + 3, feedOptions);
            contentBuffer.appendUriTree(uriTree);
            int ficheAddCount = contentBuffer.getFicheAddCount();
            int motcleAddCount = contentBuffer.getMotcleAddCount();
            if ((ficheAddCount == 0) && (motcleAddCount == 0)) {
                return;
            }
            openTag("entry");
            addSimpleElement("title", getEntryTitle(ficheAddCount, motcleAddCount));
            addSimpleElement("updated", ScrutariDBName.toGmtIsoFormat(date));
            addSimpleElement("id", feedId + "/" + ScrutariDBName.toNameFormat(date));
            startOpenTag("content");
            addAttribute("type", "xhtml");
            endOpenTag();
            startOpenTag("div");
            addNameSpaceAttribute(null, "http://www.w3.org/1999/xhtml");
            endOpenTag();
            append(contentBuffer.flush());
            closeTag("div");
            closeTag("content");
            closeTag("entry");
        }

        private String getEntryTitle(int ficheAddCount, int motcleAddCount) {
            StringBuilder buf = new StringBuilder();
            buf.append(sctServer.getName());
            buf.append(" + ");
            buf.append(ficheAddCount);
            if (motcleAddCount > 0) {
                buf.append(" / +");
                buf.append(motcleAddCount);
            }
            return buf.toString();
        }

        private String getFeedId() {
            StringBuilder buf = new StringBuilder();
            buf.append("scrutarifeed:");
            buf.append("/");
            buf.append((String) sctContext.getContextObject(SctContextConstants.UUID_NAME));
            buf.append("/");
            buf.append(sctServer.getName());
            return buf.toString();
        }

    }


    private static class ContentBuffer {

        private int motcleAddCount;
        private int ficheAddCount;
        private StringBuilder stringBuilder;
        private DataAccess dataAccess;
        private int workingLangInteger;
        private BaseBuffer baseBuffer;
        private int indentValue;
        private FeedOptions feedOptions;

        private ContentBuffer(DataAccess dataAccess, int workingLangInteger, int indentValue, FeedOptions feedOptions) {
            this.dataAccess = dataAccess;
            this.workingLangInteger = workingLangInteger;
            this.baseBuffer = new BaseBuffer(dataAccess, workingLangInteger, indentValue + 1, feedOptions);
            this.indentValue = indentValue;
            this.feedOptions = feedOptions;
        }

        private int getFicheAddCount() {
            return ficheAddCount;
        }

        private int getMotcleAddCount() {
            return motcleAddCount;
        }

        private String flush() {
            String s = stringBuilder.toString();
            stringBuilder = null;
            motcleAddCount = 0;
            ficheAddCount = 0;
            return s;
        }

        public void appendUriTree(URITree uriTree) {
            stringBuilder = new StringBuilder();
            motcleAddCount = 0;
            ficheAddCount = 0;
            XMLWriter xmlWriter = XMLUtils.getXMLWriter(stringBuilder, indentValue);
            try {
                int baseCount = uriTree.getBaseCount();
                for (int i = 0; i < baseCount; i++) {
                    BaseNode baseNode = uriTree.getBaseNode(i);
                    BaseURI baseURI = baseNode.getBaseURI();
                    BaseData baseData = (BaseData) dataAccess.getScrutariData(baseURI);
                    if (baseData == null) {
                        continue;
                    }
                    int checkedLangInteger = baseData.checkLangUi(workingLangInteger);
                    baseBuffer.appendBase(baseNode, checkedLangInteger);
                    int baseFicheAddCount = baseBuffer.getFicheAddCount();
                    int baseMotcleAddCount = baseBuffer.getMotcleAddCount();
                    String baseString = baseBuffer.flush();
                    if ((baseFicheAddCount == 0) && (baseMotcleAddCount == 0)) {
                        continue;
                    }
                    boolean active = baseNode.isActive();
                    ficheAddCount = ficheAddCount + baseFicheAddCount;
                    motcleAddCount = motcleAddCount + baseMotcleAddCount;
                    xmlWriter.openTag("p");
                    insertOpenTag(xmlWriter, active);
                    xmlWriter.escape(LibelleUtils.getLibelleString(baseData, BaseData.INTITULE_LONG, workingLangInteger, baseURI.getBaseName()));
                    insertCloseTag(xmlWriter, active);
                    xmlWriter.closeTag("p", false, true);
                    xmlWriter.openTag("ul");
                    xmlWriter.append(baseString);
                    xmlWriter.closeTag("ul");
                }
            } catch (IOException ioe) {
            }
        }

    }


    private static class BaseBuffer {

        private DataAccess dataAccess;
        private StringBuilder stringBuilder;
        private int motcleAddCount;
        private int ficheAddCount;
        private CorpusBuffer corpusBuffer;
        private ThesaurusBuffer thesaurusBuffer;
        private int workingLangInteger;
        private int indentValue;
        private FeedOptions feedOptions;

        private BaseBuffer(DataAccess dataAccess, int workingLangInteger, int indentValue, FeedOptions feedOptions) {
            this.dataAccess = dataAccess;
            corpusBuffer = new CorpusBuffer(dataAccess, indentValue + 1);
            thesaurusBuffer = new ThesaurusBuffer(dataAccess, indentValue + 1);
            this.workingLangInteger = workingLangInteger;
            this.indentValue = indentValue;
            this.feedOptions = feedOptions;

        }

        private int getFicheAddCount() {
            return ficheAddCount;
        }

        private int getMotcleAddCount() {
            return motcleAddCount;
        }

        private String flush() {
            String s = stringBuilder.toString();
            stringBuilder = null;
            motcleAddCount = 0;
            ficheAddCount = 0;
            return s;
        }

        private void appendBase(BaseNode baseNode, int checkedLangInteger) {
            stringBuilder = new StringBuilder();
            motcleAddCount = 0;
            ficheAddCount = 0;
            XMLWriter xmlWriter = XMLUtils.getXMLWriter(stringBuilder, indentValue);
            try {
                int corpusCount = baseNode.getCorpusCount();
                for (int j = 0; j < corpusCount; j++) {
                    CorpusNode corpusNode = baseNode.getCorpusNode(j);
                    CorpusURI corpusURI = corpusNode.getCorpusURI();
                    CorpusData corpusData = (CorpusData) dataAccess.getScrutariData(corpusURI);
                    if (corpusData == null) {
                        continue;
                    }
                    corpusBuffer.appendFiches(corpusNode, checkedLangInteger);
                    int ficheCount = corpusBuffer.getAddCount();
                    String ficheString = corpusBuffer.flush();
                    if (ficheCount == 0) {
                        continue;
                    }
                    boolean corpusActive = corpusNode.isActive();
                    ficheAddCount = ficheAddCount + ficheCount;
                    xmlWriter.openTag("li");
                    xmlWriter.openTag("p");
                    insertOpenTag(xmlWriter, corpusActive);
                    xmlWriter.escape(LibelleUtils.getLibelleString(corpusData, CorpusData.INTITULECORPUS_FIELDNUMBER, workingLangInteger, corpusURI.getCorpusName()));
                    insertCloseTag(xmlWriter, corpusActive);
                    xmlWriter.closeTag("p", false, true);
                    xmlWriter.openTag("ul");
                    xmlWriter.append(ficheString);
                    xmlWriter.closeTag("ul");
                    xmlWriter.closeTag("li");
                }
                if (!feedOptions.isOnlyWithFiches()) {
                    int thesaurusCount = baseNode.getThesaurusCount();
                    for (int j = 0; j < thesaurusCount; j++) {
                        ThesaurusNode thesaurusNode = baseNode.getThesaurusNode(j);
                        ThesaurusURI thesaurusURI = thesaurusNode.getThesaurusURI();
                        ThesaurusData thesaurusData = (ThesaurusData) dataAccess.getScrutariData(thesaurusURI);
                        if (thesaurusData == null) {
                            continue;
                        }
                        thesaurusBuffer.appendMotcles(thesaurusNode);
                        int motcleCount = thesaurusBuffer.getAddCount();
                        String motcleString = thesaurusBuffer.flush();
                        if (motcleCount == 0) {
                            continue;
                        }
                        boolean thesaurusActive = thesaurusNode.isActive();
                        motcleAddCount = motcleAddCount + motcleCount;
                        xmlWriter.openTag("li");
                        xmlWriter.openTag("p");
                        insertOpenTag(xmlWriter, thesaurusActive);
                        xmlWriter.escape(LibelleUtils.getLibelleString(thesaurusData, workingLangInteger, thesaurusURI.getThesaurusName()));
                        insertCloseTag(xmlWriter, thesaurusActive);
                        xmlWriter.closeTag("p", false, true);
                        xmlWriter.openTag("ul");
                        xmlWriter.append(motcleString);
                        xmlWriter.closeTag("ul");
                        xmlWriter.closeTag("li");
                    }
                }
            } catch (IOException ioe) {
            }
        }

    }


    private static class CorpusBuffer {

        private DataAccess dataAccess;
        private StringBuilder stringBuilder;
        private int addCount;
        private int indentValue;

        private CorpusBuffer(DataAccess dataAccess, int indentValue) {
            this.dataAccess = dataAccess;
            this.indentValue = indentValue;
        }

        private int getAddCount() {
            return addCount;
        }

        private String flush() {
            String s = stringBuilder.toString();
            stringBuilder = null;
            addCount = 0;
            return s;
        }

        private void appendFiches(CorpusNode corpusNode, int checkedLangInteger) {
            stringBuilder = new StringBuilder();
            addCount = 0;
            XMLWriter xmlWriter = XMLUtils.getXMLWriter(stringBuilder, indentValue);
            try {
                int ficheCount = corpusNode.getFicheCount();
                for (int k = 0; k < ficheCount; k++) {
                    FicheNode ficheNode = corpusNode.getFicheNode(k);
                    FicheURI ficheURI = ficheNode.getFicheURI();
                    FicheData ficheData = (FicheData) dataAccess.getScrutariData(ficheURI);
                    if (ficheData == null) {
                        continue;
                    }
                    addCount++;
                    xmlWriter.openTag("li");
                    String href = ficheData.getHref(checkedLangInteger);
                    if (href != null) {
                        xmlWriter.startOpenTag("a");
                        xmlWriter.addAttribute("href", href);
                        xmlWriter.endOpenTag(false);
                    }
                    String titre = ficheData.getTitre();
                    if (titre != null) {
                        xmlWriter.escape(titre);
                    } else {
                        xmlWriter.escape(ficheURI.getFicheId());
                    }
                    if (href != null) {
                        xmlWriter.closeTag("a", false, false);
                    }
                    xmlWriter.closeTag("li", false, true);
                }
            } catch (IOException ioe) {
            }
        }

    }


    private static class ThesaurusBuffer {

        private DataAccess dataAccess;
        private StringBuilder stringBuilder;
        private int addCount;
        private int indentValue;

        private ThesaurusBuffer(DataAccess dataAccess, int indentValue) {
            this.dataAccess = dataAccess;
            this.indentValue = indentValue;
        }

        private int getAddCount() {
            return addCount;
        }

        private String flush() {
            String s = stringBuilder.toString();
            stringBuilder = null;
            addCount = 0;
            return s;
        }

        private void appendMotcles(ThesaurusNode thesaurusNode) {
            stringBuilder = new StringBuilder();
            addCount = 0;
            XMLWriter xmlWriter = XMLUtils.getXMLWriter(stringBuilder, indentValue);
            try {
                int motcleCount = thesaurusNode.getMotcleCount();
                for (int k = 0; k < motcleCount; k++) {
                    MotcleNode motcleNode = thesaurusNode.getMotcleNode(k);
                    MotcleURI motcleURI = motcleNode.getMotcleURI();
                    MotcleData motcleData = (MotcleData) dataAccess.getScrutariData(motcleURI);
                    if (motcleData == null) {
                        continue;
                    }
                    addCount++;
                    xmlWriter.openTag("li");
                    int libelleCount = motcleData.getLibelleCount();
                    for (int l = 0; l < libelleCount; l++) {
                        if (l > 0) {
                            xmlWriter.append(" / ");
                        }
                        Libelle libelle = motcleData.getLibelleByIndex(l);
                        xmlWriter.escape(libelle.getLibelleString());
                    }
                    xmlWriter.closeTag("li", false, true);
                }
            } catch (IOException ioe) {
            }
        }

    }

    private static void insertOpenTag(XMLWriter xmlWriter, boolean active) throws IOException {
        if (active) {
            xmlWriter.openTag("strong", false, false);
        } else {
            xmlWriter.openTag("em", false, false);
        }
    }

    private static void insertCloseTag(XMLWriter xmlWriter, boolean active) throws IOException {
        if (active) {
            xmlWriter.closeTag("strong", false, false);
        } else {
            xmlWriter.closeTag("em", false, false);
        }
    }

}
