/* ScrutariLib_DB - Copyright (c) 2005-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.db;

import java.util.ArrayList;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.datauri.BaseURI;
import net.mapeadores.scrutari.data.BaseData;
import net.mapeadores.scrutari.data.CorpusData;
import net.mapeadores.scrutari.data.ThesaurusData;
import net.mapeadores.scrutari.data.MotcleData;
import net.mapeadores.scrutari.data.FicheData;
import net.mapeadores.util.collections.LinkedIntHashMap;
import net.mapeadores.util.primitives.PrimUtils;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.exceptions.SwitchException;
import java.util.Map;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;


/**
 * Dans toutes les fonctions d'ajout, on suppose qu'il n'y a pas de doublons.
 *
 * @author Vincent Calame
 */
public class DataAccessProviderBuilder {

    private LinkedIntHashMap baseDataMap = new LinkedIntHashMap();
    private LinkedIntHashMap corpusDataMap = new LinkedIntHashMap();
    private LinkedIntHashMap thesaurusDataMap = new LinkedIntHashMap();
    private Map<ScrutariDataURI, Integer> codeByScrutariDataURIMap = new HashMap<ScrutariDataURI, Integer>();
    private DataCacheWriter dataCacheWriter;
    private Map<Integer, IndexationBuffer> indexationBufferMap = new HashMap<Integer, IndexationBuffer>();

    public DataAccessProviderBuilder(DataCacheWriter dataCacheWriter) {
        this.dataCacheWriter = dataCacheWriter;
    }

    public DataAccessProvider toDataAccessProvider() {
        Map<Long, IndexationCodeList> indexationMap = new HashMap<Long, IndexationCodeList>();
        for (Map.Entry<Integer, IndexationBuffer> entry : indexationBufferMap.entrySet()) {
            int elementCode = entry.getKey();
            IndexationBuffer indexationBuffer = entry.getValue();
            indexationBuffer.convertInto(elementCode, indexationMap);
        }
        DataCacheReaderFactory scrutariCacheReaderFactory = dataCacheWriter.endCacheWrite();
        InternalDataAccessProvider dataAccessProvider = new InternalDataAccessProvider(scrutariCacheReaderFactory);
        dataAccessProvider.baseDataMap = baseDataMap;
        dataAccessProvider.corpusDataMap = corpusDataMap;
        dataAccessProvider.thesaurusDataMap = thesaurusDataMap;
        dataAccessProvider.codeByScrutariDataURIMap = codeByScrutariDataURIMap;
        dataAccessProvider.dataCacheReaderFactory = scrutariCacheReaderFactory;
        dataAccessProvider.indexationMap = indexationMap;
        dataAccessProvider.codecorpusList = CodeListUtils.toCodeList(corpusDataMap.getKeyArray());
        dataAccessProvider.codethesaurusList = CodeListUtils.toCodeList(thesaurusDataMap.getKeyArray());
        indexationBufferMap = null;
        return dataAccessProvider;
    }

    public boolean containsBase(BaseURI baseURI) {
        Integer itg = codeByScrutariDataURIMap.get(baseURI);
        if (itg == null) {
            return false;
        }
        return baseDataMap.containsKey(itg.intValue());
    }


    private static class InternalDataAccessProvider implements DataAccessProvider {

        private LinkedIntHashMap baseDataMap;
        private LinkedIntHashMap corpusDataMap;
        private LinkedIntHashMap thesaurusDataMap;
        private Map<ScrutariDataURI, Integer> codeByScrutariDataURIMap;
        private CodeList codecorpusList;
        private CodeList codethesaurusList;
        private Map<Long, IndexationCodeList> indexationMap;
        private DataCacheReaderFactory dataCacheReaderFactory;

        private InternalDataAccessProvider(DataCacheReaderFactory dataCacheReaderFactory) {
            this.dataCacheReaderFactory = dataCacheReaderFactory;
        }

        public DataAccess openDataAccess() {
            return new InternalDataAccess();
        }


        private class InternalDataAccess implements DataAccess {

            private DataCacheReader dataCacheReader;

            private InternalDataAccess() {
            }

            public CodeList getCodecorpusList() {
                return codecorpusList;
            }

            public CodeList getCodethesaurusList() {
                return codethesaurusList;
            }

            public int getBaseCount() {
                return baseDataMap.size();
            }

            public int getCorpusCount() {
                return corpusDataMap.size();
            }

            public int getThesaurusCount() {
                return thesaurusDataMap.size();
            }

            public int getCode(ScrutariDataURI scrutariDataURI) {
                Integer itg = codeByScrutariDataURIMap.get(scrutariDataURI);
                if (itg == null) {
                    return -1;
                }
                return itg.intValue();
            }

            public Object getScrutariData(ScrutariDataURI scrutariDataURI) {
                Integer itg = codeByScrutariDataURIMap.get(scrutariDataURI);
                if (itg == null) {
                    return null;
                }
                int code = itg.intValue();
                short type = scrutariDataURI.getType();
                switch (type) {
                    case ScrutariDataURI.BASEURI_TYPE:
                        return baseDataMap.get(code);
                    case ScrutariDataURI.CORPUSURI_TYPE:
                        return corpusDataMap.get(code);
                    case ScrutariDataURI.THESAURUSURI_TYPE:
                        return thesaurusDataMap.get(code);
                    case ScrutariDataURI.FICHEURI_TYPE:
                        return getFicheDataByCode(code);
                    case ScrutariDataURI.MOTCLEURI_TYPE:
                        return getMotcleDataByCode(code);
                    default:
                        throw new SwitchException("Unknown type: " + type);
                }
            }

            public BaseData getBaseDataByCode(int codebase) {
                return (BaseData) baseDataMap.get(codebase);
            }

            public BaseData getBaseDataByIndex(int index) {
                return (BaseData) baseDataMap.getValueAt(index);
            }

            public CorpusData getCorpusDataByCode(int codecorpus) {
                return (CorpusData) corpusDataMap.get(codecorpus);
            }

            public CorpusData getCorpusDataByIndex(int index) {
                return (CorpusData) corpusDataMap.getValueAt(index);
            }

            public ThesaurusData getThesaurusDataByCode(int codethesaurus) {
                return (ThesaurusData) thesaurusDataMap.get(codethesaurus);
            }

            public ThesaurusData getThesaurusDataByIndex(int index) {
                return (ThesaurusData) thesaurusDataMap.getValueAt(index);
            }

            public MotcleData getMotcleDataByCode(int codemotcle) {
                if (dataCacheReader == null) {
                    dataCacheReader = dataCacheReaderFactory.newReader();
                }
                return dataCacheReader.getMotcleData(codemotcle);
            }

            public FicheData getFicheDataByCode(int codefiche) {
                if (dataCacheReader == null) {
                    dataCacheReader = dataCacheReaderFactory.newReader();
                }
                return dataCacheReader.getFicheData(codefiche);
            }

            public int getFicheDateIntegerByCode(int codefiche) {
                FicheData ficheData = getFicheDataByCode(codefiche);
                if (ficheData != null) {
                    return ficheData.getDateInteger();
                } else {
                    return DateInteger.NULL;
                }
            }

            public IndexationCodeList getIndexationCodeList(int elementCode, int subsetCode) {
                long lg = PrimUtils.toLong(elementCode, subsetCode);
                IndexationCodeList indexationCodeList = indexationMap.get(lg);
                if (indexationCodeList == null) {
                    return CodeListUtils.EMPTY_INDEXATIONCODELIST;
                }
                return indexationCodeList;
            }

            public void close() {
                if (dataCacheReader != null) {
                    dataCacheReader.close();
                }
            }

        }

    }

    public void addBaseData(BaseData baseData) {
        int codebase = baseData.getCodeBase();
        baseDataMap.put(codebase, baseData);
        codeByScrutariDataURIMap.put(baseData.getBaseURI(), codebase);
    }

    public void addCorpusData(CorpusData corpusData) {
        int codecorpus = corpusData.getCodeCorpus();
        corpusDataMap.put(codecorpus, corpusData);
        codeByScrutariDataURIMap.put(corpusData.getCorpusURI(), codecorpus);
    }

    public void addThesaurusData(ThesaurusData thesaurusData) {
        int codethesaurus = thesaurusData.getCodeThesaurus();
        thesaurusDataMap.put(codethesaurus, thesaurusData);
        codeByScrutariDataURIMap.put(thesaurusData.getThesaurusURI(), codethesaurus);
    }

    public void addMotcleData(MotcleData motcleData) {
        int codemotcle = motcleData.getCodeMotcle();
        codeByScrutariDataURIMap.put(motcleData.getMotcleURI(), codemotcle);
        dataCacheWriter.cacheMotcleData(motcleData);
    }

    public void addFicheData(FicheData ficheData) {
        int codefiche = ficheData.getCodeFiche();
        codeByScrutariDataURIMap.put(ficheData.getFicheURI(), codefiche);
        dataCacheWriter.cacheFicheData(ficheData);
    }

    public void addIndexation(int codefiche, int codecorpus, int codemotcle, int codethesaurus, float ponderation) {
        IndexationBuffer ficheIndexationBuffer = getOrCreateIndexationBuffer(codefiche);
        ficheIndexationBuffer.add(codemotcle, codethesaurus, ponderation);
        IndexationBuffer motcleIndexationBuffer = getOrCreateIndexationBuffer(codemotcle);
        motcleIndexationBuffer.add(codefiche, codecorpus, ponderation);
    }

    private IndexationBuffer getOrCreateIndexationBuffer(int elementCode) {
        IndexationBuffer indexationBuffer = indexationBufferMap.get(elementCode);
        if (indexationBuffer == null) {
            indexationBuffer = new IndexationBuffer();
            indexationBufferMap.put(elementCode, indexationBuffer);
        }
        return indexationBuffer;
    }


    private static class IndexationBuffer {

        private Map<Integer, MutableIndexationCodeList> listMap = new HashMap<Integer, MutableIndexationCodeList>();

        private IndexationBuffer() {
        }

        private void add(int elementCode, int subsetCode, float ponderation) {
            MutableIndexationCodeList codeListBuffer = listMap.get(subsetCode);
            if (codeListBuffer == null) {
                codeListBuffer = new MutableIndexationCodeList();
                listMap.put(subsetCode, codeListBuffer);
            }
            codeListBuffer.add(elementCode, ponderation);
        }

        private void convertInto(int elementCode, Map<Long, IndexationCodeList> indexationMap) {
            for (Map.Entry<Integer, MutableIndexationCodeList> entry : listMap.entrySet()) {
                int subsetCode = entry.getKey();
                MutableIndexationCodeList codeListBuffer = entry.getValue();
                long lg = PrimUtils.toLong(elementCode, subsetCode);
                indexationMap.put(lg, codeListBuffer.toIndexationCodeList());
            }
        }

    }


    private static class MutableIndexationCodeList {

        private List<Integer> elementList = new ArrayList<Integer>();
        private List<Float> ponderationList = new ArrayList<Float>();
        private Set<Integer> elementSet = new HashSet<Integer>();

        private void add(int elementCode, float ponderation) {
            Integer itg = new Integer(elementCode);
            if (!elementSet.contains(itg)) {
                elementSet.add(itg);
                elementList.add(itg);
                ponderationList.add(ponderation);
            }
        }

        private IndexationCodeList toIndexationCodeList() {
            return CodeListUtils.toIndexatoinCodeList(elementList, ponderationList);
        }

    }

}
