/* ScrutariLib_Data - Copyright (c) 2005-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.data;

import net.scrutari.datauri.FicheURI;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.text.StringUtils;
import java.util.List;
import java.util.ArrayList;
import java.io.Serializable;
import java.io.IOException;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;


/**
 *
 * @author  Vincent Calame
 */
public final class FicheData implements Serializable {

    private int codefiche;
    private int codecorpus;
    private int codebase;
    private int langInteger;
    private int dateInteger;
    private String titre;
    private String soustitre;
    private String hrefPart1;
    private String hrefPart2;
    private String[] complements;
    private short type;
    private FicheURI ficheURI;
    private String ficheIcon;
    private String latitude;
    private String longitude;

    /*
     * Constructeur utilisé par FicheDataTupleBinding pour l'insertion dans la base de Berkeley.
     */
    private FicheData() {
    }

    private FicheData(FicheURI ficheURI, int codefiche, int codecorpus, int codebase, int langInteger, int dateInteger, String titre, String soustitre, String href, String[] complements, short type, String ficheIcon, String latitude, String longitude) {
        this.codecorpus = codecorpus;
        this.codefiche = codefiche;
        this.codebase = codebase;
        this.langInteger = langInteger;
        this.dateInteger = dateInteger;
        this.titre = titre;
        this.soustitre = soustitre;
        parseHref(href);
        this.complements = complements;
        this.type = type;
        this.ficheURI = ficheURI;
        this.ficheIcon = ficheIcon;
        this.latitude = latitude;
        this.longitude = longitude;
    }

    public int getCodeFiche() {
        return codefiche;
    }

    public int getCodeCorpus() {
        return codecorpus;
    }

    public int getCodeBase() {
        return codebase;
    }

    public int getLangInteger() {
        return langInteger;
    }

    public int getComplementMaxNumber() {
        if (complements == null) {
            return 0;
        }
        return complements.length;
    }

    public FicheURI getFicheURI() {
        return ficheURI;
    }

    /**
     * Peut avoir la valeur nulle mais n'est pas de longueur nulle ou composé d'espace blanc.
     */
    public String getTitre() {
        return titre;
    }

    /**
     * Peut avoir la valeur nulle mais n'est pas de longueur nulle ou composé d'espace blanc.
     */
    public String getSoustitre() {
        return soustitre;
    }

    public String getHref(int langUI) {
        if (hrefPart2 == null) {
            return hrefPart1;
        }
        StringBuilder buf = new StringBuilder(hrefPart1.length() + hrefPart2.length() + 10);
        buf.append(hrefPart1);
        LangInteger.appendString(buf, langUI);
        buf.append(hrefPart2);
        return buf.toString();
    }

    public String getRawHref() {
        if (hrefPart2 == null) {
            return hrefPart1;
        }
        return hrefPart1 + "$LANGUI" + hrefPart2;
    }

    public int getDateInteger() {
        return dateInteger;
    }

    public short getType() {
        return type;
    }

    public String getFicheIcon() {
        return ficheIcon;
    }

    public String getLatitude() {
        return latitude;
    }

    public String getLongitude() {
        return longitude;
    }

    /**
     * Attention, les compléments sont numérotés à partir de 1.
     */
    public String getComplementByNumber(int i) {
        if (complements == null) {
            throw new IndexOutOfBoundsException();
        }
        return complements[i - 1];
    }

    public static FicheData fromPrimitives(PrimitivesReader primitivesReader) throws IOException {
        FicheData ficheData = new FicheData();
        ficheData.ficheURI = (FicheURI) URIPrimitivesUtils.readScrutariDataURI(primitivesReader);
        ficheData.codefiche = primitivesReader.readInt();
        ficheData.codecorpus = primitivesReader.readInt();
        ficheData.langInteger = primitivesReader.readInt();
        ficheData.dateInteger = primitivesReader.readInt();
        ficheData.codebase = primitivesReader.readInt();
        ficheData.type = primitivesReader.readShort();
        ficheData.titre = primitivesReader.readString();
        ficheData.soustitre = primitivesReader.readString();
        ficheData.hrefPart1 = primitivesReader.readString();
        ficheData.hrefPart2 = primitivesReader.readString();
        ficheData.ficheIcon = primitivesReader.readString();
        ficheData.latitude = primitivesReader.readString();
        ficheData.longitude = primitivesReader.readString();
        int length = primitivesReader.readInt();
        if (length > 0) {
            String[] comp = new String[length];
            for (int i = 0; i < length; i++) {
                comp[i] = primitivesReader.readString();
            }
            ficheData.complements = comp;
        }
        return ficheData;
    }

    public static void toPrimitives(FicheData ficheData, PrimitivesWriter primitivesWriter) throws IOException {
        URIPrimitivesUtils.writeScrutariDataURI(ficheData.ficheURI, primitivesWriter);
        primitivesWriter.writeInt(ficheData.codefiche);
        primitivesWriter.writeInt(ficheData.codecorpus);
        primitivesWriter.writeInt(ficheData.langInteger);
        primitivesWriter.writeInt(ficheData.dateInteger);
        primitivesWriter.writeInt(ficheData.codebase);
        primitivesWriter.writeShort(ficheData.type);
        primitivesWriter.writeString(ficheData.titre);
        primitivesWriter.writeString(ficheData.soustitre);
        primitivesWriter.writeString(ficheData.hrefPart1);
        primitivesWriter.writeString(ficheData.hrefPart2);
        primitivesWriter.writeString(ficheData.ficheIcon);
        primitivesWriter.writeString(ficheData.latitude);
        primitivesWriter.writeString(ficheData.longitude);
        if (ficheData.complements == null) {
            primitivesWriter.writeInt(-1);
        } else {
            int length = ficheData.complements.length;
            primitivesWriter.writeInt(length);
            for (int i = 0; i < length; i++) {
                primitivesWriter.writeString(ficheData.complements[i]);
            }
        }
    }

    private void parseHref(String href) {
        if (href == null) {
            return;
        }
        int idx = href.indexOf("$LANGUI");
        if (idx == -1) {
            hrefPart1 = href;
            hrefPart2 = null;
        } else {
            hrefPart1 = href.substring(0, idx);
            hrefPart2 = href.substring(idx + 7);
        }
    }


    public static class Builder {

        private FicheURI ficheURI;
        private int langInteger = LangInteger.UNDETERMINED;
        private int dateInteger = DateInteger.NULL;
        private int complementMaxNumber = 0;
        private String titre;
        private String soustitre;
        private String href;
        private String ficheIcon;
        private String latitude;
        private String longitude;
        private short type = TypeConstants.UNKNOWN_TYPE;
        private List<String> complementsList;

        /**
         * Construit une instance de FicheInfoBuffer pour la fiche d'URI <code>ficheURI</code> en précisant le
         * nombre maximum de champs complémentaires qu'est susceptible de contenir la fiche.
         */
        public Builder(FicheURI ficheURI, int complementMaxNumber) {
            this.ficheURI = ficheURI;
            this.complementMaxNumber = complementMaxNumber;
            if (complementMaxNumber > 0) {
                complementsList = new ArrayList<String>();
            }
        }

        public void setType(short type) {
            this.type = type;
        }

        /**
         * Modifie l'entier de la langue de la fiche
         */
        public void setLangInteger(int langInteger) {
            this.langInteger = langInteger;
        }

        /**
         * Modifie l'entier de la date de la fiche.
         */
        public void setDateInteger(int dateInteger) {
            this.dateInteger = dateInteger;
        }

        /**
         * Modifie le titre de la fiche.
         */
        public void setTitre(String titre) {
            this.titre = checkString(titre);
        }

        /**
         * Modifie le sous-titre de la fiche
         */
        public void setSoustitre(String soustitre) {
            this.soustitre = checkString(soustitre);
        }

        /**
         * Modifie l'hyperlien de la fiche.
         */
        public void setHref(String href) {
            this.href = checkString(href);
        }

        public void setFicheIcon(String ficheIcon) {
            this.ficheIcon = checkString(ficheIcon);
        }

        public void setGeoloc(String latitude, String longitude) {
            latitude = checkString(latitude);
            longitude = checkString(longitude);
            if ((latitude != null) && (longitude != null)) {
                this.latitude = latitude;
                this.longitude = longitude;
            }
        }

        /**
         * Retourne l'entier de la langue de la fiche.
         */
        public int getLangInteger() {
            return langInteger;
        }

        /**
         * Retourne l'entier de la date de la fiche.
         */
        public int getDateInteger() {
            return dateInteger;
        }

        /**
         * Retourne l'URI de la fiche.
         */
        public FicheURI getFicheURI() {
            return ficheURI;
        }

        /**
         * Ajoute la valeur d'un champ complémentaire. L'ordre d'ajout des champs détermine l'ordre
         * final des champs dans l'instance de FicheData issue de l'objet FicheInfoBuffer.
         * Cette méthode retourne false si le nombre maximum de champs est atteint.
         */
        public boolean addComplement(String s) {
            if (complementMaxNumber == 0) {
                return false;
            }
            if (complementsList.size() == complementMaxNumber) {
                return false;
            }
            complementsList.add(checkString(s));
            return true;
        }

        /**
         * Construit une instance de FicheData à partir des données de l'objet FicheInfoBuffer.
         */
        public FicheData toFicheData(int codefiche, short defaultType, String hrefParent, int codecorpus, int codebase) {
            if (type == TypeConstants.UNKNOWN_TYPE) {
                type = defaultType;
            }
            if ((href != null) && (hrefParent != null)) {
                if ((!(href.startsWith("http://"))) || (!(href.startsWith("https://")))) {
                    href = hrefParent + href;
                }
            }
            String[] complements = null;
            if (complementMaxNumber > 0) {
                complements = new String[complementMaxNumber];
                for (int i = 0; i < complementsList.size(); i++) {
                    complements[i] = complementsList.get(i);
                }
            }
            return new FicheData(ficheURI, codefiche, codecorpus, codebase, langInteger, dateInteger, titre, soustitre, href, complements, type, ficheIcon, latitude, longitude);
        }

        private String checkString(String s) {
            if (s == null) {
                return null;
            }
            s = StringUtils.cleanString(s);
            if (s.length() == 0) {
                return null;
            }
            return s;
        }

    }

}
