/* ScrutariLib_SearchEngine - Copyright (c) 2011-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.searchengine;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import net.mapeadores.scrutari.lexie.FieldNumbers;
import net.mapeadores.scrutari.searchengine.pertinence.FichePertinence;
import net.mapeadores.scrutari.searchengine.pertinence.MotclePertinence;
import net.mapeadores.util.primitives.DateInteger;


/**
 *
 * @author Vincent Calame
 */
public class FicheSortEngine {

    private static final int NOTE_MAXIMALE = 143;
    private static final DescendingComparator COMPARATOR = new DescendingComparator();
    private GlobalSearchOptions globalSearchOptions;
    private InternalFicheSortGroup defaultFicheSortGroup;
    private SortedMap<Integer, InternalFicheSortGroup> ficheSortGroupByClasseMap;
    private int globalFicheCount = 0;
    private int currentDate = DateInteger.current();
    private int currentAnnee = DateInteger.getAnnee(currentDate);
    private int searchTokenCount;
    private int searchTokenMask;

    public FicheSortEngine(int searchTokenCount, int searchTokenMask, GlobalSearchOptions globalSearchOptions) {
        this.searchTokenCount = searchTokenCount;
        this.searchTokenMask = searchTokenMask;
        this.globalSearchOptions = globalSearchOptions;
        PertinencePonderation globalPertinencePonderation = globalSearchOptions.getPertinencePonderation();
        int corpusClasseCount = globalSearchOptions.getCorpusClasseCount();
        if (corpusClasseCount == 0) {
            defaultFicheSortGroup = new InternalFicheSortGroup(0, globalPertinencePonderation);
        } else {
            ficheSortGroupByClasseMap = new TreeMap<Integer, InternalFicheSortGroup>();
            for (int i = 0; i < corpusClasseCount; i++) {
                CorpusClasse corpusClasse = globalSearchOptions.getCorpusClasse(i);
                ficheSortGroupByClasseMap.put(corpusClasse.getRank(), new InternalFicheSortGroup(corpusClasse.getRank(), globalPertinencePonderation));
            }
        }
    }

    public void addFichePertinence(FichePertinence fichePertinence) {
        InternalFicheSortGroup ficheSortGroup = getFicheSortGroup(fichePertinence.getCodeCorpus());
        ficheSortGroup.addFichePertinence(fichePertinence);
        globalFicheCount++;
    }

    public int getFicheCount() {
        return globalFicheCount;
    }

    public void sort() {
        if (defaultFicheSortGroup != null) {
            defaultFicheSortGroup.sort();
        } else {
            for (InternalFicheSortGroup ficheSortGroup : ficheSortGroupByClasseMap.values()) {
                ficheSortGroup.sort();
            }
        }
    }

    public FicheSortGroup[] getFicheSortGroupArray() {
        if (globalFicheCount == 0) {
            return null;
        }
        if (defaultFicheSortGroup != null) {
            FicheSortGroup[] result = new FicheSortGroup[1];
            result[0] = defaultFicheSortGroup;
            return result;
        } else {
            List<InternalFicheSortGroup> list = new ArrayList<InternalFicheSortGroup>();
            for (InternalFicheSortGroup ficheSortGroup : ficheSortGroupByClasseMap.values()) {
                if (!ficheSortGroup.isEmpty()) {
                    list.add(ficheSortGroup);
                }
            }
            int size = list.size();
            FicheSortGroup[] result = new FicheSortGroup[size];
            int p = size - 1;
            for (InternalFicheSortGroup ficheSortGroup : list) {
                result[p] = ficheSortGroup;
                p--;
            }
            return result;
        }
    }

    private InternalFicheSortGroup getFicheSortGroup(int codecorpus) {
        if (defaultFicheSortGroup != null) {
            return defaultFicheSortGroup;
        }
        int corpusClasseRank = globalSearchOptions.getCorpusClasseRank(codecorpus);
        return ficheSortGroupByClasseMap.get(corpusClasseRank);
    }


    private class InternalFicheSortGroup implements FicheSortGroup {

        private Map<Integer, PertinenceSort> pertinenceSortMap = new HashMap<Integer, PertinenceSort>();
        private float raretePoids;
        private float fraicheurPoids;
        private float positionPoids;
        private PertinenceSort groupPertinenceSort = new PertinenceSort();
        private int corpusClasseRank;
        private int ficheCount = 0;

        private InternalFicheSortGroup(int corpusClasseRank, PertinencePonderation pertinencePonderation) {
            this.corpusClasseRank = corpusClasseRank;
            this.raretePoids = pertinencePonderation.getRarete();
            this.fraicheurPoids = pertinencePonderation.getFraicheur();
            this.positionPoids = pertinencePonderation.getPosition();
        }

        public FichePertinence[] toFichePertinenceArray() {
            return groupPertinenceSort.toFichePertinenceArray();
        }

        public int getCorpusClasseRank() {
            return corpusClasseRank;
        }

        public boolean isEmpty() {
            return (ficheCount == 0);
        }

        private void addFichePertinence(FichePertinence fichePertinence) {
            ficheCount++;
            float fraicheur = getFraicheur(fichePertinence.getDateInteger());
            float position = getPosition(fichePertinence);
            float pertinenceRate = fraicheurPoids * fraicheur + positionPoids * position + 0.00001f;
            fichePertinence.setPertinenceRate(pertinenceRate);
            int codecorpus = fichePertinence.getCodeCorpus();
            PertinenceSort pertinenceSort = pertinenceSortMap.get(codecorpus);
            if (pertinenceSort == null) {
                pertinenceSort = new PertinenceSort();
                pertinenceSortMap.put(codecorpus, pertinenceSort);
            }
            pertinenceSort.addFichePertinence(fichePertinence);
        }

        private void sort() {
            for (PertinenceSort pertinenceSort : pertinenceSortMap.values()) {
                FichePertinence[] array = pertinenceSort.sort(raretePoids);
                int length = array.length;
                for (int i = 0; i < length; i++) {
                    groupPertinenceSort.addFichePertinence(array[i]);
                }
            }
        }

    }

    /**
     * la fraicheur est calculee avec la formule :
     * 1 - x/(x+6) où x est la valeur absolue de la différence entre l'année de la fiche
     * et l'année en cours. Cela vaut 0 si la date est indéterminée.
     */
    private float getFraicheur(int dateInteger) {
        if (dateInteger == DateInteger.NULL) {
            return 0;
        }
        int annee = DateInteger.getAnnee(dateInteger);
        float x = (float) Math.abs(annee - currentAnnee);
        return 1 - (x / (x + 6));
    }

    private float getPosition(FichePertinence fichePertinence) {
        int result = getPositionInFiche(fichePertinence) + getPositionInMotcle(fichePertinence);
        return ((float) result) / NOTE_MAXIMALE;
    }

    private int getPositionInFiche(FichePertinence fichePertinence) {
        int fieldNote = getFieldNote(fichePertinence);
        int debutNote = getFieldMinimalIndexNote(fichePertinence.getMinimalBeginIndex());
        return fieldNote + debutNote;
    }

    /*
     * Note en fonction de la position des mots dans les champs de recherche, de 0 à 40
     */
    private int getFieldNote(FichePertinence fichePertinence) {
        int minimalFieldNumber = fichePertinence.getMinimalFieldNumber();
        boolean complet = (fichePertinence.getFicheCompletudeMask() == searchTokenMask);
        if (complet) {
            boolean completSoustitre = (fichePertinence.getSearchTokenCountInSoustitre() == searchTokenCount);
            if (minimalFieldNumber == FieldNumbers.TITRE_FIELDNUMBER) {
                if (fichePertinence.getSearchTokenCountInTitre() == searchTokenCount) {
                    return 100;
                } else if (completSoustitre) {
                    return 90;
                } else {
                    return 60;
                }
            }
            if (minimalFieldNumber == FieldNumbers.SOUSTITRE_FIELDNUMBER) {
                if (completSoustitre) {
                    return 80;
                } else {
                    return 50;
                }
            }
            return 40;
        } else {
            if (minimalFieldNumber == FieldNumbers.TITRE_FIELDNUMBER) {
                return 30;
            }
            if (minimalFieldNumber == FieldNumbers.SOUSTITRE_FIELDNUMBER) {
                return 20;
            }
            if (minimalFieldNumber > 0) {
                return 10;
            }
            return 0;
        }
    }

    /**
     * Note de 0 à 3, en fonction de la position initiale du premier mot
     */
    private int getFieldMinimalIndexNote(int minimalIndex) {
        if (minimalIndex == 0) {
            return 4;
        }
        if (minimalIndex < 50) {
            return 3;
        }
        if (minimalIndex < 150) {
            return 2;
        }
        if (minimalIndex < 300) {
            return 1;
        }
        return 0;
    }

    private int getPositionInMotcle(FichePertinence fichePertinence) {
        boolean complet = (fichePertinence.getMotcleCompletudeMask() == searchTokenMask);
        if (complet) {
            MotclePertinence max = fichePertinence.getMotclePertinenceMax();
            if (max.getCompletudeMask() == searchTokenMask) {
                if (fichePertinence.getPonderationMax() > 1) {
                    return 40;
                } else {
                    return 35;
                }
            } else {
                return 25;
            }
        } else {
            return 15;
        }
    }


    private static class PertinenceSort {

        private int count = 0;
        private SortedMap<Float, FichePertinencePack> sortedFichePertinenceMap = new TreeMap<Float, FichePertinencePack>(COMPARATOR);

        PertinenceSort() {
        }

        private void addFichePertinence(FichePertinence fichePertinence) {
            count++;
            float pertinenceRate = fichePertinence.getPertinenceRate();
            FichePertinencePack fichePertinencePack = sortedFichePertinenceMap.get(pertinenceRate);
            if (fichePertinencePack == null) {
                fichePertinencePack = new FichePertinencePack();
                sortedFichePertinenceMap.put(pertinenceRate, fichePertinencePack);
            }
            fichePertinencePack.add(fichePertinence);
        }

        private FichePertinence[] toFichePertinenceArray() {
            FichePertinence[] result = new FichePertinence[count];
            int p = 0;
            for (FichePertinencePack fichePertinencePack : sortedFichePertinenceMap.values()) {
                int ficheCount = fichePertinencePack.getFicheCount();
                for (int i = 0; i < ficheCount; i++) {
                    FichePertinence fichePertinence = fichePertinencePack.getFichePertinence(i);
                    result[p] = fichePertinence;
                    p++;
                }
            }
            return result;
        }

        private FichePertinence[] sort(float poidsRarete) {
            FichePertinence[] result = new FichePertinence[count];
            int p = 0;
            float floatCount = (float) count;
            for (FichePertinencePack fichePertinencePack : sortedFichePertinenceMap.values()) {
                int ficheCount = fichePertinencePack.getFicheCount();
                for (int i = 0; i < ficheCount; i++) {
                    FichePertinence fichePertinence = fichePertinencePack.getFichePertinence(i);
                    float pertinenceRate = fichePertinence.getPertinenceRate();
                    float rarete = (float) Math.pow((floatCount - p) / floatCount, 2);
                    pertinenceRate = pertinenceRate + rarete * poidsRarete;
                    if (pertinenceRate > 1) pertinenceRate = 1;
                    fichePertinence.setPertinenceRate(pertinenceRate);
                    result[p] = fichePertinence;
                    p++;
                }
            }
            return result;
        }

    }


    private static class FichePertinencePack {

        private List<FichePertinence> fichePertinenceList = new ArrayList<FichePertinence>();

        private FichePertinencePack() {
        }

        private void add(FichePertinence fichePertinence) {
            fichePertinenceList.add(fichePertinence);
        }

        public int getFicheCount() {
            return fichePertinenceList.size();
        }

        public FichePertinence getFichePertinence(int i) {
            return fichePertinenceList.get(i);
        }

    }


    private static class DescendingComparator implements Comparator<Float> {

        public int compare(Float o1, Float o2) {
            float f1 = o1.floatValue();
            float f2 = o2.floatValue();
            if (f1 > f2) {
                return -1;
            }
            if (f1 < f2) {
                return 1;
            }
            return 0;
        }

    }

}
