/* ScrutariLib_SearchEngine - Copyright (c) 2005-2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.searchengine.pertinence;

import net.mapeadores.util.collections.LinkedIntHashMap;
import net.mapeadores.util.text.SubstringPosition;


/**
 * Classe qui stocke la pertinence d'un mot-clé par rapport à une recherche.
 *
 * @author Vincent Calame
 */
public class MotclePertinence implements Comparable {

    private int codemotcle;
    private int codethesaurus;
    private int positiontextemin = 9999;
    private LinkedIntHashMap pertinencebyLangArrayMap = new LinkedIntHashMap();
    private int longueurmin = 9999;
    private float maxCoverRate = 0;
    private int degrecompletude = 0;
    private int completudeMask = 0;
    private int sequenceTokenCount;
    private boolean hasLangUI = false;

    /**
     * Construit une instance de MotclePertinence pour un mot-clé de la base avec
     * l'indication du nombre de mots dans la sequence de recherche.
     */
    public MotclePertinence(int codemotcle, int codethesaurus, int sequenceTokenCount) {
        this.codemotcle = codemotcle;
        this.codethesaurus = codethesaurus;
        this.sequenceTokenCount = sequenceTokenCount;
    }

    /**
     * Retourne le code du mot-clé.
     */
    public int getCodeMotcle() {
        return codemotcle;
    }

    /**
     * Retourne le code du thésaurus du mot-clé.
     */
    public int getCodeThesaurus() {
        return codethesaurus;
    }

    /**
     * Retourne le nombre de langues pour lesquelles il a été trouvé un mot de la séquence de recherche
     *dans les libellés du mot-clé.
     */
    public int getConcernedLangCount() {
        return pertinencebyLangArrayMap.size();
    }

    /**
     * Retourne la i-ème langue concernée.
     */
    public int getConcernedLangInteger(int i) {
        return pertinencebyLangArrayMap.getKeyAt(i);
    }

    /**
     * Retourne un tableau des positions des mots de la séquence dans le libellé de la i-ième langue concernée.
     * Ce tableau a au moins un élément (sinon la langue ne serait pas concernée). Sa longueur est celle
     * du nombre de mots dans la séquence. Il contient des éléments nuls à l'index d'un mot de la séquence
     * qui ne se trouve pas dans le libellé. Ce tableau comprend cependant au moins un élément non null.
     */
    public SubstringPosition[] getSubstringPositionByTokenArray(int i) {
        return ((PertinenceByLang) pertinencebyLangArrayMap.getValueAt(i)).getSourcePositionByTokenArray();
    }

    /**
     * Retourne le « masque de complétude » de l'objet PertinenceMotcle.
     * Le masque indique les bits des mots de la séquence de recherche pour lesquels
     *le mot-clé est pertinent.
     */
    public int getCompletudeMask() {
        return completudeMask;
    }

    public int getSearchTokenCount() {
        return degrecompletude;
    }

    /**
     * Réécriture de la méthode <em>equals</em> afin que this.equals(obj) == (this.compareTo(obj) == 0)
     */
    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if ((!(obj instanceof MotclePertinence)) || (obj == null)) {
            return false;
        }
        return (compareTo(obj) == 0);
    }

    /**
     * Réécriture de la méthode <em>hashCode</em> pour que si this.equals(obj), alors
     * this.hashCode() == obj.hashCode().
     */
    @Override
    public int hashCode() {
        return codemotcle;
    }

    /**
     * Implémentation de l'interface Comparable. C'est cette méthode qui permet
     * de trier les instances de PertinenceMotcle entre elles. L'ordre des mots-clés est
     * l'inverse de la pertinence (i.e. : les premiers mots-clés sont les plus pertinents). 
     */
    public int compareTo(Object o) {
        MotclePertinence pert = (MotclePertinence) o;
        if (codemotcle == pert.codemotcle) {
            return 0;
        }
        if (degrecompletude > pert.degrecompletude) {
            return -1;
        }
        if (degrecompletude < pert.degrecompletude) {
            return 1;
        }
        if (hasLangUI != pert.hasLangUI) {
            if (hasLangUI) {
                return -1;
            } else {
                return 1;
            }
        }
        if (positiontextemin < pert.positiontextemin) {
            return -1;
        }
        if (positiontextemin > pert.positiontextemin) {
            return 1;
        }
        if (maxCoverRate > pert.maxCoverRate) {
            return -1;
        }
        if (maxCoverRate < pert.maxCoverRate) {
            return 1;
        }
        if (longueurmin < pert.longueurmin) {
            return -1;
        }
        if (longueurmin > pert.longueurmin) {
            return 1;
        }
        if (completudeMask > pert.completudeMask) {
            return -1;
        }
        if (completudeMask < pert.completudeMask) {
            return 1;
        }
        if (codemotcle < pert.codemotcle) {
            return -1;
        }
        if (codemotcle > pert.codemotcle) {
            return 1;
        }
        return 0;
    }

    /**
     * Compare le degré de « complétude » de deux instances 
     */
    public int compareCompletude(MotclePertinence pert) {
        if (degrecompletude > pert.degrecompletude) {
            return -1;
        }
        if (degrecompletude < pert.degrecompletude) {
            return 1;
        }
        return 0;
    }

    /*
     * Précise la pertinence du mot-clé pour un mot de la séquence à la position tokenIndex
     * en précisant la position du mot dans le libellé et le taux de couverture.
     */
    public void addPertinence(int tokenIndex, int langInteger, SubstringPosition sourcePosition, float coverRate) {
        int tokenIndexBit = (int) Math.pow(2, tokenIndex);
        PertinenceByLang pertinenceByLang = getPertinenceByLang(langInteger);
        if ((completudeMask & tokenIndexBit) == 0) {
            degrecompletude++;
        }
        completudeMask = (completudeMask | tokenIndexBit);
        if (sourcePosition != null) {
            pertinenceByLang.setSourcePosition(tokenIndex, tokenIndexBit, sourcePosition);
            positiontextemin = Math.min(sourcePosition.getBeginIndex(), positiontextemin);
            longueurmin = Math.min(sourcePosition.getLength(), longueurmin);
            maxCoverRate = Math.max(maxCoverRate, coverRate);
        }
    }

    public void finalizePertinence(int uiLangInteger) {
        PertinenceByLang pertinenceByLang = (PertinenceByLang) pertinencebyLangArrayMap.get(uiLangInteger);
        if (pertinenceByLang != null) {
            hasLangUI = true;
            if (pertinencebyLangArrayMap.size() == 1) {
                return;
            }
            if (pertinenceByLang.isComplete()) {
                pertinencebyLangArrayMap.clear();
                pertinencebyLangArrayMap.put(uiLangInteger, pertinenceByLang);
            }
        }
    }

    private PertinenceByLang getPertinenceByLang(int langInteger) {
        PertinenceByLang pertinenceByLang = (PertinenceByLang) pertinencebyLangArrayMap.get(langInteger);
        if (pertinenceByLang == null) {
            pertinenceByLang = new PertinenceByLang(langInteger);
            pertinencebyLangArrayMap.put(langInteger, pertinenceByLang);
        }
        return pertinenceByLang;
    }


    private class PertinenceByLang {

        int langInteger;
        SubstringPosition[] sourcePositionByTokenArray;
        int completudebyLangMask = 0;

        PertinenceByLang(int langInteger) {
            this.langInteger = langInteger;
            sourcePositionByTokenArray = new SubstringPosition[sequenceTokenCount];
        }

        SubstringPosition[] getSourcePositionByTokenArray() {
            return sourcePositionByTokenArray;
        }

        void setSourcePosition(int tokenIndex, int tokenIndexBit, SubstringPosition sourcePosition) {
            SubstringPosition currentPosition = sourcePositionByTokenArray[tokenIndex];
            if ((currentPosition == null) || (sourcePosition.getBeginIndex() < currentPosition.getBeginIndex())) {
                sourcePositionByTokenArray[tokenIndex] = sourcePosition;
            }
            completudebyLangMask = completudebyLangMask | tokenIndexBit;
        }

        boolean isComplete() {
            return (completudebyLangMask == completudeMask);
        }

    }

}
