/* ScrutariLib_SearchEngine - Copyright (c) 2008 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.searchengine.pertinence;

import net.mapeadores.scrutari.searchengine.GlobalSearchOptions;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import java.util.Comparator;


/**
 *
 * @author Vincent Calame
 */
public class PertinenceComparator implements Comparator<FichePertinence> {

    private int searchTokenCount;
    private int searchTokenMask;
    private GlobalSearchOptions globalSearchOptions;

    public PertinenceComparator(int searchTokenCount, int searchTokenMask) {
        this.searchTokenMask = searchTokenMask;
        this.searchTokenCount = searchTokenCount;
    }

    public int compare(FichePertinence fp1, FichePertinence fp2) {
        if (fp1 == fp2) {
            return 0;
        }

        /*
         * Cas particulier des corpus de notions, placés en premier.
         */
        int classeCorpusStep = getClasseCorpusStep(fp1, fp2);
        if (classeCorpusStep != 0) {
            return classeCorpusStep;
        }


        /*
         * Comparaison particulière lorsque tous les mots de la recherche sont 
         * compris dans un ou plusieurs mots-clés.
         */
        int motcleCompletudeStep = getMotcleCompletudeStep(fp1, fp2);
        if (motcleCompletudeStep != 0) {
            return motcleCompletudeStep;
        }

        /*
         * Comparaison sur le nombre de mots dans le titre
         */
        int titreStep = Comparaison.searchTokenCountInTitre(fp1, fp2);
        if (titreStep != 0) {
            return titreStep;
        }

        /*
         * Comparaison sur le nombre de mots dans les mots-clés
         */
        int motcleCompletudeDegreStep = Comparaison.searchTokenCountInMotscles(fp1, fp2);
        if (motcleCompletudeDegreStep != 0) {
            return motcleCompletudeDegreStep;
        }

        /*
         * Comparaison des plus fortes pondérations relevées.
         */
        int ponderationMaxStep = Comparaison.ponderationMax(fp1, fp2);
        if (ponderationMaxStep != 0) {
            return ponderationMaxStep;
        }

        /* Comparaison sur le nombre de mots dans le sous-titre*/
        int soustitreStep = Comparaison.searchTokenCountInSoustitre(fp1, fp2);
        if (soustitreStep != 0) {
            return soustitreStep;
        }


        /*
         * Comparaison sur l'année.
         */
        int anneeStep = Comparaison.annee(fp1, fp2);
        if (anneeStep != 0) {
            return anneeStep;
        }


        /* À ce stade, vu, les test précédents, si une fiche a des mots dans le titre ou dans le sous-titre
         * (ce que dit la condition (fp1.getMinimalFieldNumber() < 0)), l'autre a le même nombre de mots dans le même champ.
         * La différence se fait
         * sur la position minimale au sein du champ.
         */
        if (fp1.getMinimalFieldNumber() < 0) {
            int minimalBeginIndexStep = Comparaison.minimalBeginIndex(fp1, fp2);
            if (minimalBeginIndexStep != 0) {
                return minimalBeginIndexStep;
            }
        }

        /* Compare la pertinence des meilleurs mots-clés (s'ils existent)*/
        int motclePertinenceMaxStep = Comparaison.motclePertinenceMax(fp1, fp2);
        if (motclePertinenceMaxStep != 0) {
            return motclePertinenceMaxStep;
        }


        /*À ce stade il ne reste plus que la comparaison sur les compléments */
        int minimalComplementFieldStep = Comparaison.minimalFieldNumber(fp1, fp2);
        if (minimalComplementFieldStep != 0) {
            return minimalComplementFieldStep;
        }
        int minimalComplementBeginIndexStep = Comparaison.minimalBeginIndex(fp1, fp2);
        if (minimalComplementBeginIndexStep != 0) {
            return minimalComplementBeginIndexStep;
        }

        /*
         * Comparaison sur la date complète
         */
        int dateStep = Comparaison.date(fp1, fp2);
        if (dateStep != 0) {
            return dateStep;
        }


        /* Comparaison ultime */
        int codeStep = Comparaison.code(fp1, fp2);
        if (codeStep != 0) {
            return codeStep;
        }

        throw new ShouldNotOccurException("Il ne devrait pas y avoir d'instance de FichePertinence avec le même code");
    }

    /*
     * étape 1 = vérification que tous les mots-clés couvrent le champ de recherche
     * si oui, le critère est la qualité du mot-clé
     */
    private int getMotcleCompletudeStep(FichePertinence fp1, FichePertinence fp2) {
        int mc1 = fp1.getMotcleCompletudeMask();
        int mc2 = fp2.getMotcleCompletudeMask();
        if (mc1 == searchTokenMask) {
            if (mc2 != searchTokenMask) {
                return -1;
            } else {
                if (searchTokenCount > 1) {
                    int maxSearchTokenCount = Comparaison.maxSearchTokenCountInMotcle(fp1, fp2);
                    if (maxSearchTokenCount != 0) {
                        return maxSearchTokenCount;
                    }
                }
                return Comparaison.ponderationMax(fp1, fp2);
            }
        } else if (mc2 == searchTokenMask) {
            return 1;
        } else {
            return 0;
        }
    }

    private int getClasseCorpusStep(FichePertinence fp1, FichePertinence fp2) {
        if (globalSearchOptions == null) {
            return 0;
        }
        int crp1 = fp1.getCodeCorpus();
        int crp2 = fp2.getCodeCorpus();
        if (crp1 == crp2) {
            return 0;
        }
        int cv1 = globalSearchOptions.getCorpusClasseRank(crp1);
        int cv2 = globalSearchOptions.getCorpusClasseRank(crp2);
        if (cv1 == cv2) {
            return 0;
        }
        if (cv1 > cv2) {
            return -1;
        } else {
            return 1;
        }
    }

    public void setGlobalSearchOptions(GlobalSearchOptions globalSearchOptions) {
        this.globalSearchOptions = globalSearchOptions;
    }

}
