/* ScrutariLib_Xml - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.xml;

import net.scrutari.datauri.BaseURI;
import net.mapeadores.scrutari.data.BaseData;
import net.mapeadores.scrutari.data.CorpusData;
import net.mapeadores.scrutari.data.ThesaurusData;
import net.mapeadores.scrutari.db.ScrutariDB;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.scrutari.xml.xmlpull.CorpusHandler;
import net.mapeadores.scrutari.xml.xmlpull.ThesaurusHandler;
import net.mapeadores.scrutari.xml.xmlpull.IndexationGroupHandler;
import net.mapeadores.util.xml.xmlpull.DefaultElementHandler;
import net.mapeadores.util.xml.xmlpull.SimpleElementHandler;
import net.mapeadores.util.xml.xmlpull.XmlPullElementHandler;
import net.mapeadores.util.xml.xmlpull.LibelleElementHandler;
import java.util.Map;
import java.util.HashMap;
import java.io.IOException;
import net.mapeadores.scrutari.xml.xmlpull.XmlPullUtils;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;


/**
 *
 * @author Vincent Calame
 */
public class XmlPullReader implements ErrorKeys {

    private ScrutariDB.Builder scrutariDBBuilder;
    private ScrutariXmlErrorHandler scrutariXmlErrorHandler;
    private BaseData.Builder baseDataBuilder;
    private Parameters parameters;
    public static final int AUTHORITY_UUID = 1;
    public static final int BASE_NAME = 2;
    public static final int BASE_ICON = 3;
    public static final int LANG = 4;

    public XmlPullReader(ScrutariDB.Builder scrutariDBBuilder, ScrutariXmlErrorHandler scrutariXmlErrorHandler) {
        this.scrutariDBBuilder = scrutariDBBuilder;
        this.scrutariXmlErrorHandler = scrutariXmlErrorHandler;
        parameters = new Parameters(scrutariDBBuilder, scrutariXmlErrorHandler);

    }

    public BaseURI process(XmlPullParser xpp) throws XmlPullParserException, IOException {
        baseDataBuilder = null;
        RootHandler rootHandler = null;
        try {
            int eventType = xpp.getEventType();
            do {
                if (eventType == xpp.START_TAG) {
                    if (rootHandler == null) {
                        if (xpp.getName().equals("base")) {
                            rootHandler = new RootHandler();
                        } else {
                            scrutariXmlErrorHandler.addSevereLog(SEVERE_XML_BASEROOT);
                            break;
                        }
                    } else {
                        Map<String, String> map = null;
                        int count = xpp.getAttributeCount();
                        if (count > 0) {
                            map = new HashMap<String, String>();
                            for (int i = 0; i < count; i++) {
                                map.put(xpp.getAttributeName(i), xpp.getAttributeValue(i));
                            }
                        }
                        rootHandler.processStartElement(xpp.getName(), map);
                    }
                } else if (eventType == xpp.END_TAG) {
                    rootHandler.processEndElement(xpp.getName());
                } else if (eventType == xpp.TEXT) {
                    if (rootHandler != null) {
                        rootHandler.processText(xpp.getText());
                    }
                }
                eventType = xpp.next();
            } while (eventType != xpp.END_DOCUMENT);
            if (rootHandler != null) {
                if (baseDataBuilder == null) {
                    scrutariXmlErrorHandler.addSevereLog(SEVERE_XML_BASE_MISSINGMETADATA);
                    return null;
                } else {
                    scrutariDBBuilder.addBase(baseDataBuilder);
                    return baseDataBuilder.getBaseURI();
                }
            } else {
                return null;
            }
        } catch (InterruptionException ie) {
            return null;
        }
    }

    public void flush() {
        scrutariDBBuilder.setIndexation(parameters.getIndexationBuilder());
    }


    private class RootHandler extends DefaultElementHandler {

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (baseDataBuilder == null) {
                if (!tagname.equals("base-metadata")) {
                    scrutariXmlErrorHandler.addSevereLog(SEVERE_XML_BASE_MISSINGMETADATA);
                    throw new InterruptionException();
                }
                return new MetadataHandler();
            } else if (tagname.equals("corpus")) {
                try {
                    return new CorpusHandler(attributeMap, parameters);
                } catch (IllegalArgumentException iae) {
                    return null;
                }

            } else if (tagname.equals("thesaurus")) {
                try {
                    return new ThesaurusHandler(attributeMap, parameters);
                } catch (IllegalArgumentException iae) {
                    return null;
                }
            } else if (tagname.equals("indexation-group")) {
                try {
                    return new IndexationGroupHandler(attributeMap, parameters);
                } catch (IllegalArgumentException iae) {
                    return null;
                }

            } else {
                return null;
            }
        }

        @Override
        public void endSubElement(XmlPullElementHandler elementHandler) {
            if (elementHandler instanceof CorpusHandler) {
                CorpusData.Builder corpusDataBuilder = ((CorpusHandler) elementHandler).getCorpusDataBuilder();
                if (corpusDataBuilder != null) {
                    baseDataBuilder.addCorpusDataBuilder(corpusDataBuilder);
                }
            } else if (elementHandler instanceof ThesaurusHandler) {
                ThesaurusData.Builder thesaurusDataBuilder = ((ThesaurusHandler) elementHandler).getThesaurusDataBuilder();
                if (thesaurusDataBuilder != null) {
                    baseDataBuilder.addThesaurusDataBuilder(thesaurusDataBuilder);
                }
            } else if (elementHandler instanceof IndexationGroupHandler) {
            }
        }

        BaseData.Builder getBaseDataBuilder() {
            return baseDataBuilder;
        }

    }


    private class MetadataHandler extends DefaultElementHandler {

        String authorityuuid = null;
        String baseName = null;

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map attributeMap) {
            if (tagname.equals("authority-uuid")) {
                return new SimpleElementHandler(AUTHORITY_UUID);
            }
            if (tagname.equals("base-name")) {
                return new SimpleElementHandler(BASE_NAME);
            }
            initBaseDataBuilder();
            if (tagname.equals("base-icon")) {
                return new SimpleElementHandler(BASE_ICON);
            } else if (tagname.equals("intitule-short")) {
                return new IntituleHandler(BaseData.INTITULE_SHORT);
            } else if (tagname.equals("intitule-long")) {
                return new IntituleHandler(BaseData.INTITULE_LONG);
            } else if (tagname.equals("langs-ui")) {
                return new LangUiHandler();
            }
            return null;
        }

        @Override
        public void endSimpleSubElement(int type, String text) {
            if (text.length() == 0) {
                text = null;
            }
            switch (type) {
                case AUTHORITY_UUID:
                    authorityuuid = text;
                    break;
                case BASE_NAME:
                    baseName = text;
                    break;
                case BASE_ICON:
                    baseDataBuilder.setBaseIcon(text);
                    break;
            }
        }

        private void initBaseDataBuilder() {
            if (baseDataBuilder != null) {
                return;
            }
            if ((authorityuuid == null) || (baseName == null)) {
                if (authorityuuid == null) {
                    scrutariXmlErrorHandler.addSevereLog(SEVERE_XML_AUTHORITYUUID);
                }
                if (baseName == null) {
                    scrutariXmlErrorHandler.addSevereLog(SEVERE_XML_NOMBASE);
                }
                throw new InterruptionException();
            }
            BaseURI baseURI = new BaseURI(authorityuuid, baseName);
            if (scrutariDBBuilder.containsBase(baseURI)) {
                scrutariXmlErrorHandler.addSevereLog(SEVERE_DATABASE_DOUBLE, baseURI);
                throw new InterruptionException();
            }
            baseDataBuilder = new BaseData.Builder(baseURI);
            parameters.setBaseURI(baseURI);
        }

    }


    private class IntituleHandler extends DefaultElementHandler {

        short type;

        IntituleHandler(short type) {
            this.type = type;
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (tagname.equals("lib")) {
                try {
                    return new LibelleElementHandler(attributeMap);
                } catch (LangIntegerException lie) {
                    scrutariXmlErrorHandler.addWarningLog(WARNING_XML_BASE_LANGERROR, baseDataBuilder.getBaseURI());
                    return null;
                }
            }
            return null;
        }

        @Override
        public void endSubElement(XmlPullElementHandler elementHandler) {
            Libelle libelle = XmlPullUtils.testLibelle(elementHandler);
            if (libelle != null) {
                baseDataBuilder.addLibelle(type, libelle);
            }

        }

    }


    private class LangUiHandler extends DefaultElementHandler {

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map attributeMap) {
            if (tagname.equals("lang")) {
                return new SimpleElementHandler(LANG);
            }
            return null;
        }

        @Override
        public void endSimpleSubElement(int type, String text) {
            try {
                int langInteger = LangInteger.parse(text);
                baseDataBuilder.addLangUI(langInteger);
            } catch (LangIntegerException ile) {
                scrutariXmlErrorHandler.addWarningLog(WARNING_XML_BASE_LANGERROR, baseDataBuilder.getBaseURI());
            }
        }

    }


    private static class InterruptionException extends RuntimeException {
    }

}
