/* SctServer - Copyright (c) 2008-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.admin;

import net.mapeadores.scrutari.searchengine.CorpusClasse;
import net.scrutari.datauri.CorpusURI;
import net.mapeadores.scrutari.searchengine.GlobalSearchOptions;
import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.request.RequestInfo;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.searchengine.PertinencePonderation;
import net.mapeadores.util.collections.IntHashMap;
import net.mapeadores.util.conditions.ConditionUtils;
import net.mapeadores.util.conditions.TextTest;


/**
 *
 * @author Vincent Calame
 */
public class OptionsManager {

    private SctServer sctServer;
    private InternalGlobalSearchOptions internalGlobalSearchOptions = new InternalGlobalSearchOptions();
    private boolean withClasses = false;
    private CorpusClasse[] corpusClasseArray;
    private SearchLogDeny[] searchLogDenyArray;

    public OptionsManager(SctServer sctServer) {
        this.sctServer = sctServer;
    }

    public void reloadCorpusClasseArray(CorpusClasse[] corpusClasseArray) {
        withClasses = false;
        this.corpusClasseArray = null;
        DataAccess dataAccess = sctServer.getScrutariDB().openDataAccess();
        int defaultClasseValue = 0;
        IntHashMap codeMap = null;
        if (corpusClasseArray != null) {
            this.corpusClasseArray = corpusClasseArray;
            withClasses = true;
            codeMap = new IntHashMap();
            int length = corpusClasseArray.length;
            for (int i = 0; i < length; i++) {
                CorpusClasse corpusClasse = corpusClasseArray[i];
                String name = corpusClasse.getName();
                if (name == null) {
                    defaultClasseValue = corpusClasse.getRank();
                } else {
                    int count = corpusClasse.getCorpusURICount();
                    for (int j = 0; j < count; j++) {
                        CorpusURI corpusURI = corpusClasse.getCorpusURI(j);
                        int codecorpus = dataAccess.getCode(corpusURI);
                        if (codecorpus > 0) {
                            codeMap.put(codecorpus, corpusClasse);
                        }
                    }
                }
            }
        }
        dataAccess.close();
        internalGlobalSearchOptions.reload(codeMap, defaultClasseValue);
    }

    public void setGlobalPertinencePonderation(float fraicheur, float rarete, float position) {
        if (fraicheur < 0) {
            fraicheur = 0;
        }
        if (rarete < 0) {
            rarete = 0;
        }
        if (position < 0) {
            position = 0;
        }
        float somme = fraicheur + rarete + position;
        if ((somme != 0) && (somme != 1)) {
            fraicheur = fraicheur / somme;
            rarete = rarete / somme;
            position = position / somme;
        }
        InternalPertinencePonderation ipp = internalGlobalSearchOptions.pertinencePonderation;
        ipp.setFraicheur(fraicheur);
        ipp.setPosition(position);
        ipp.setRarete(rarete);
    }

    public void reloadSearchLogDenyArray(SearchLogDeny[] searchLogDenyArray) {
        this.searchLogDenyArray = null;
        if (searchLogDenyArray != null) {
            if (searchLogDenyArray.length != 0) {
                this.searchLogDenyArray = searchLogDenyArray;
            }
        }
    }

    public GlobalSearchOptions getGlobalSearchOptions() {
        return internalGlobalSearchOptions;
    }

    public int getSearchLogDenyCount() {
        if (searchLogDenyArray == null) {
            return 0;
        }
        return searchLogDenyArray.length;
    }

    public SearchLogDeny getSearchLogDeny(int i) {
        return searchLogDenyArray[i];
    }

    public boolean filterLog(RequestInfo requestInfo) {
        if (!requestInfo.withLog()) {
            return false;
        }
        if (searchLogDenyArray == null) {
            return true;
        }
        int length = searchLogDenyArray.length;
        if (length == 0) {
            return true;
        }
        for (int i = 0; i < length; i++) {
            SearchLogDeny logDeny = searchLogDenyArray[i];
            TextTest ipTest = logDeny.getTextTest(SearchLogDeny.IP_DENY);
            if (ipTest != null) {
                String remoteAdress = requestInfo.getRemoteAddress();
                if ((remoteAdress == null) || (remoteAdress.length() == 0)) {
                    return false;
                }
                boolean test = ConditionUtils.accept(ipTest, requestInfo.getRemoteAddress());
                if (!test) {
                    continue;
                }
            }
            TextTest siteTest = logDeny.getTextTest(SearchLogDeny.SITE_DENY);
            if (siteTest != null) {
                String site = requestInfo.getSite();
                if ((site == null) || (site.length() == 0)) {
                    return false;
                }
                boolean test = ConditionUtils.accept(siteTest, site);
                if (!test) {
                    continue;
                }
            }
            TextTest refTest = logDeny.getTextTest(SearchLogDeny.REF_DENY);
            if (refTest != null) {
                String referrer = requestInfo.getReferrer();
                if ((referrer == null) && (referrer.length() == 0)) {
                    return false;
                }
                boolean test = ConditionUtils.accept(refTest, referrer);
                if (!test) {
                    continue;
                }
            }
            return false;
        }
        return true;
    }


    private class InternalGlobalSearchOptions implements GlobalSearchOptions {

        private IntHashMap codeMap;
        private int defautClassValue;
        private InternalPertinencePonderation pertinencePonderation = new InternalPertinencePonderation();

        public boolean isWithCorpusClasse() {
            return withClasses;
        }

        public int getCorpusClasseRank(int codecorpus) {
            if (!withClasses) {
                return 0;
            }
            CorpusClasse corpusClasse = (CorpusClasse) codeMap.get(codecorpus);
            if (corpusClasse == null) {
                return defautClassValue;
            } else {
                return corpusClasse.getRank();
            }
        }

        public int getCorpusClasseCount() {
            if (!withClasses) {
                return 0;
            }
            return corpusClasseArray.length;
        }

        public CorpusClasse getCorpusClasse(int i) {
            return corpusClasseArray[i];
        }

        public CorpusClasse getCorpusClasseByRank(int rank) {
            if (!withClasses) {
                return null;
            }
            int length = corpusClasseArray.length;
            for (int i = 0; i < length; i++) {
                CorpusClasse corpusClasse = corpusClasseArray[i];
                if (corpusClasse.getRank() == rank) {
                    return corpusClasse;
                }
            }
            return null;
        }

        public PertinencePonderation getPertinencePonderation() {
            return pertinencePonderation;
        }

        private void reload(IntHashMap codeMap, int defautClassValue) {
            this.defautClassValue = defautClassValue;
            this.codeMap = codeMap;
        }

        private void set() {
        }

    }


    private static class InternalPertinencePonderation implements PertinencePonderation {

        private float rarete = 0.15f;
        private float fraicheur = 0.25f;
        private float position = 0.6f;

        public float getRarete() {
            return rarete;
        }

        public float getFraicheur() {
            return fraicheur;
        }

        public float getPosition() {
            return position;
        }

        public void setRarete(float rarete) {
            this.rarete = rarete;
        }

        public void setFraicheur(float fraicheur) {
            this.fraicheur = fraicheur;
        }

        public void setPosition(float position) {
            this.position = position;
        }

    }

}
