/* SctServer - Copyright (c) 2008 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.admin;

import fr.exemole.sctserver.storage.SctServerStorage;
import net.mapeadores.util.exceptions.NestedLibraryException;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.DateIntegerException;
import net.mapeadores.util.xml.XMLUtils;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.security.cert.X509Certificate;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;


/**
 *
 * @author Vincent Calame
 */
public class UrlCollector {

    private SctServerStorage sctServerStorage;
    private ScrutariInfo scrutariInfo;
    private short finalState;
    private CollectErrorHandler collectErrorHandler;
    private int lastUpdate;
    private URL dataUrl;

    public UrlCollector(SctServerStorage sctServerStorage, CollectErrorHandler collectErrorHandler) {
        this.sctServerStorage = sctServerStorage;
        this.collectErrorHandler = collectErrorHandler;
        try {
            SSLContext sc = SSLContext.getInstance("SSLv3");
            TrustManager[] tma = {new InternalTrustManager()};
            sc.init(null, tma, null);
            SSLSocketFactory ssf = sc.getSocketFactory();
            HttpsURLConnection.setDefaultSSLSocketFactory(ssf);
        } catch (Exception e) {
            throw new NestedLibraryException(e);
        }

    }

    public void setScrutariInfo(ScrutariInfo scrutariInfo) {
        this.scrutariInfo = scrutariInfo;
        dataUrl = null;
        finalState = ScrutariInfo.UNKNOWN_STATE;
        lastUpdate = scrutariInfo.getLastUpdate();
    }

    public short getFinalState() {
        return finalState;
    }

    public int getLastUpdate() {
        return lastUpdate;
    }

    public URL getDataURL() {
        return dataUrl;
    }

    public boolean collectData(boolean forceDataDownload) {
        boolean done = checkInfo();
        if (!done) {
            return false;
        }
        if (!forceDataDownload) {
            if (lastUpdate <= scrutariInfo.getLastUpdate()) {
                finalState = ScrutariInfo.OK_STATE;
                lastUpdate = scrutariInfo.getLastUpdate();
                return true;
            }
        }
        URLConnection con = null;
        try {
            con = dataUrl.openConnection();
            if (!testConnection(con)) {
                finalState = ScrutariInfo.DATA_BROKENURL_STATE;
                return false;
            }
        } catch (IOException ioe) {
            finalState = ScrutariInfo.DATA_NOCONNECTION_STATE;
            collectErrorHandler.warningDownload(dataUrl, ioe.getMessage());
            return false;
        }

        ByteArrayOutputStream out = new ByteArrayOutputStream();
        try {
            InputStream in = con.getInputStream();
            byte[] buffer = new byte[1024];
            int size;
            while ((size = in.read(buffer, 0, 1024)) > 0) {
                out.write(buffer, 0, size);
            }
            in.close();
            out.flush();
            out.close();
        } catch (IOException ioe) {
            finalState = ScrutariInfo.DATA_DOWNLOADERROR_STATE;
            collectErrorHandler.warningDownload(dataUrl, ioe.getMessage());
            return false;
        }
        ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(out.toByteArray());
        sctServerStorage.saveScrutariData(scrutariInfo, byteArrayInputStream);
        finalState = ScrutariInfo.OK_STATE;
        return true;
    }

    private boolean testConnection(URLConnection urlConnection) throws IOException {
        URL url = urlConnection.getURL();
        if (url.getProtocol().equals("https")) {
            try {
                HttpsURLConnection httpsCon = (HttpsURLConnection) urlConnection;
                int reponse = httpsCon.getResponseCode();
                if (reponse != HttpURLConnection.HTTP_OK) {
                    collectErrorHandler.warningDownload(url, httpsCon.getResponseMessage());
                    finalState = ScrutariInfo.DATA_BROKENURL_STATE;
                    return false;
                } else {
                    return true;
                }
            } catch (Exception e) {
                e.printStackTrace();
                return false;
            }

        } else {
            HttpURLConnection con = (HttpURLConnection) urlConnection;
            int reponse = con.getResponseCode();
            if (reponse != HttpURLConnection.HTTP_OK) {
                collectErrorHandler.warningDownload(url, con.getResponseMessage());
                finalState = ScrutariInfo.DATA_BROKENURL_STATE;
                return false;
            } else {
                return true;
            }
        }
    }

    private boolean checkInfo() {
        URL url = scrutariInfo.getUrl();
        URLConnection con = null;
        try {
            con = (HttpURLConnection) url.openConnection();
            if (!testConnection(con)) {
                finalState = ScrutariInfo.INFO_BROKENURL_STATE;
                return false;
            }
        } catch (IOException ioe) {
            finalState = ScrutariInfo.INFO_NOCONNECTION_STATE;
            collectErrorHandler.warningDownload(url, ioe.getMessage());
            return false;
        }
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder db = null;
        try {
            db = dbf.newDocumentBuilder();
        } catch (ParserConfigurationException e) {
            throw new NestedLibraryException(e);
        }
        Document doc = null;
        try {
            doc = db.parse(con.getInputStream());
        } catch (IOException ioe) {
            finalState = ScrutariInfo.INFO_DOWNLOADERROR_STATE;
            collectErrorHandler.warningDownload(url, ioe.getMessage());
            return false;
        } catch (SAXException saxe) {
            finalState = ScrutariInfo.INFO_NOTXML_STATE;
            collectErrorHandler.warningDownload(url, saxe.getMessage());
            return false;
        }
        return checkInfoDocument(doc.getDocumentElement());
    }

    private boolean checkInfoDocument(Element element_xml) {
        String pathString = null;
        String lastDateString = null;
        NodeList nodeList = element_xml.getChildNodes();
        int completeMask = 0;
        for (int i = 0; i < nodeList.getLength(); i++) {
            Node node = nodeList.item(i);
            if (node.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) node;
                String tagname = el.getTagName();
                if (tagname.equals("date")) {
                    lastDateString = XMLUtils.getData(el);
                } else if (tagname.equals("path")) {
                    pathString = XMLUtils.getData(el);
                } else if (tagname.equals("scrutaridata-url")) {
                    pathString = XMLUtils.getData(el);
                }
            }
        }
        if ((pathString == null) || (lastDateString == null)) {
            finalState = ScrutariInfo.INFO_WRONGXML_STATE;
            return false;
        }
        try {
            lastUpdate = DateInteger.parse(lastDateString);
        } catch (DateIntegerException ide) {
            finalState = ScrutariInfo.INFO_WRONGXML_STATE;
            return false;
        }
        try {
            dataUrl = buildDataUrl(pathString);
        } catch (MalformedURLException mue) {
            finalState = ScrutariInfo.INFO_WRONGXML_STATE;
            return false;
        }
        return true;
    }

    private URL buildDataUrl(String pathString) throws MalformedURLException {
        try {
            URL url = new URL(pathString);
            return url;
        } catch (MalformedURLException mue) {
            String infoUrlString = scrutariInfo.getUrl().toString();
            int idx = infoUrlString.lastIndexOf('/');
            URL url = new URL(infoUrlString.substring(0, idx + 1) + pathString);
            return url;
        }
    }


    public interface CollectErrorHandler {

        public void warningDownload(URL url, String message);

    }


    class InternalTrustManager implements javax.net.ssl.X509TrustManager {

        public void checkClientTrusted(X509Certificate[] chain, String authType) {
        }

        public void checkServerTrusted(X509Certificate[] chain, String authType) {
        }

        public X509Certificate[] getAcceptedIssuers() {
            return null;
        }

    }

}
