/* SctServer - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.conf;

import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.SctServerLog;
import fr.exemole.sctserver.conf.xml.UriCodesDOMReader;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.scrutari.db.ScrutariDataURICodeList;
import net.mapeadores.util.logging.MutableProcessLog;
import net.mapeadores.util.logging.ProcessLog;
import net.mapeadores.util.logging.ProcessLogItem;
import net.mapeadores.util.logging.ProcessURI;
import net.scrutari.datauri.ScrutariDataURI;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public class UriCodesConf {

    private UriCodesLog uriCodesLog;
    private ScrutariDataURICodeList scrutariDataURICodeList;
    private ProcessURI processURI;

    private UriCodesConf() {
    }

    public ProcessLog getProcessLog() {
        return uriCodesLog;
    }

    public ScrutariDataURICodeList getScrutariDataURICodeList() {
        return scrutariDataURICodeList;
    }

    public static UriCodesConf load(File f) throws IOException, ParserConfigurationException {
        UriCodesConf conf = new UriCodesConf();
        conf.init(f);
        return conf;
    }

    public ProcessURI getProcessURI() {
        return processURI;
    }

    private void init(File f) throws IOException, ParserConfigurationException {
        processURI = ProcessURI.fromFile(f);
        uriCodesLog = new UriCodesLog(processURI);
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        Document document = null;
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            document = db.parse(f);
        } catch (SAXException saxe) {
            uriCodesLog.saxException(saxe);
            return;
        }
        UriCodesBuilder builder = new UriCodesBuilder(uriCodesLog);
        UriCodesDOMReader domReader = new UriCodesDOMReader(builder, uriCodesLog);
        domReader.readUriCodes(document.getDocumentElement());
        scrutariDataURICodeList = builder.toScrutariDataURICodeList();
    }


    private static class UriCodesBuilder implements UriCodesDOMReader.Builder {

        private UriCodesLog uriCodesLog;
        private Map<Integer, ScrutariDataURI> codeMap = new LinkedHashMap<Integer, ScrutariDataURI>();
        private Map<ScrutariDataURI, Integer> uriMap = new HashMap<ScrutariDataURI, Integer>();

        public UriCodesBuilder(UriCodesLog uriCodesLog) {
            this.uriCodesLog = uriCodesLog;
        }

        public void addScrutariDataURI(int code, ScrutariDataURI scrutariDataURI) {
            if (codeMap.containsKey(code)) {
                uriCodesLog.codeDoublon(code);
                return;
            }
            if (uriMap.containsKey(scrutariDataURI)) {
                uriCodesLog.uriDoublon(scrutariDataURI);
                return;
            }
            codeMap.put(code, scrutariDataURI);
            uriMap.put(scrutariDataURI, code);
        }

        public ScrutariDataURICodeList toScrutariDataURICodeList() {
            int size = codeMap.size();
            if (size == 0) {
                return null;
            }
            int[] codeArray = new int[size];
            ScrutariDataURI[] uriArray = new ScrutariDataURI[size];
            int p = 0;
            for (Map.Entry<Integer, ScrutariDataURI> entry : codeMap.entrySet()) {
                codeArray[p] = entry.getKey();
                uriArray[p] = entry.getValue();
                p++;
            }
            return new ArrayScrutariDataURICodeList(codeArray, uriArray);
        }

    }


    private static class ArrayScrutariDataURICodeList implements ScrutariDataURICodeList {

        private int[] codeArray;
        private ScrutariDataURI[] uriArray;

        public ArrayScrutariDataURICodeList(int[] codeArray, ScrutariDataURI[] uriArray) {
            this.codeArray = codeArray;
            this.uriArray = uriArray;
        }

        public int getCodeCount() {
            return codeArray.length;
        }

        public int getCode(int i) {
            return codeArray[i];
        }

        public ScrutariDataURI getScrutariDataURI(int i) {
            return uriArray[i];
        }

    }


    private static class UriCodesLog extends MutableProcessLog implements UriCodesDOMReader.ErrorHandler {

        public UriCodesLog(ProcessURI processURI) {
            setCurrentURI(processURI);
        }

        public void saxException(SAXException saxe) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.MALFORMED_XML_CATEGORY);
            logItem.setKey(SctServerLog.SEVERE_DATA_BASEFILE_XML);
            logItem.setObject(saxe.getMessage());
        }

        public void codeDoublon(int code) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.WARNING_CATEGORY);
            logItem.setKey("codeDoublon");
            logItem.setObject(new Integer(code));
        }

        public void uriDoublon(ScrutariDataURI scrutariDataURI) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.WARNING_CATEGORY);
            logItem.setKey("uriDoublon");
            logItem.setObject(scrutariDataURI);
        }

        public void unknownTagWarning(String tagName) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.WARNING_CATEGORY);
            logItem.setKey("unknownTag");
            logItem.setObject(tagName);
        }

        public void emptyAttributeError(String tagName, String attributeName) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.SEVERE_CATEGORY);
            logItem.setKey("emptyAttribute");
            logItem.setObject(tagName + "@" + attributeName);
        }

        public void notPositiveIntegerAttributeError(String tagName, String attributeName, String attributeValue) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.SEVERE_CATEGORY);
            logItem.setKey("notPositiveIntegerAttribute");
            logItem.setObject(tagName + "@" + attributeName + "=" + attributeValue);
        }

        public void emptyElementError(String tagName) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.SEVERE_CATEGORY);
            logItem.setKey("emptyElement");
            logItem.setObject(tagName);
        }

        public void uriSyntaxError(String uriString) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.WARNING_CATEGORY);
            logItem.setKey("uriSyntaxError");
            logItem.setObject(uriString);
        }

    }

}
