/* SctServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.conf.xml;

import net.scrutari.datauri.CorpusURI;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.datauri.URIParseException;
import net.scrutari.datauri.URIParser;
import net.mapeadores.scrutari.searchengine.CorpusClasse;
import net.mapeadores.scrutari.searchengine.PertinencePonderation;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleUtils;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class CorpusClassesDOMReader {

    private Builder builder;
    private ErrorHandler errorHandler;

    public CorpusClassesDOMReader(Builder builder, ErrorHandler errorHandler) {
        this.builder = builder;
        this.errorHandler = errorHandler;
    }

    public void readCorpusClasses(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if ((tagName.equals("classes-defs")) || (tagName.equals("classe-def-list"))) {
                    addDefs(element);
                }

            }
        }
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("corpus-list")) {
                    addCorpusList(element);
                } else if (tagName.equals("ponderation")) {
                    setPonderation(element);
                } else if ((!tagName.equals("classes-defs")) && (!tagName.equals("classe-def-list"))) {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }
    }

    private void setPonderation(Element element_xml) {
        InternalPertinencePonderation pertinencePonderation = new InternalPertinencePonderation();
        pertinencePonderation.setFraicheur(getFloat(element_xml, "fraicheur"));
        pertinencePonderation.setRarete(getFloat(element_xml, "rarete"));
        pertinencePonderation.setPosition(getFloat(element_xml, "position"));
        builder.setGlobalPertinencePonderation(pertinencePonderation);
    }

    private float getFloat(Element element_xml, String attribute) {
        String vale = element_xml.getAttribute(attribute);
        if (vale.length() == 0) {
            return 0f;
        }
        try {
            float f = Float.parseFloat(vale);
            return f;
        } catch (NumberFormatException nfe) {
            return 0;
        }
    }

    private void addDefs(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("classe-def")) {
                    addDef(element);
                } else {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }
    }

    private void addDef(Element element_xml) {
        String name = element_xml.getAttribute("name");
        if (name.length() == 0) {
            errorHandler.emptyAttributeError(element_xml.getTagName(), "name");
            return;
        }
        if (name.equals("_default")) {
            name = null;
        }
        CorpusClasse corpusClasse = builder.addClasse(name);
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("lib")) {
                    try {
                        Libelle libelle = LibelleUtils.readLib(element);
                        if (libelle != null) {
                            builder.putLibelle(corpusClasse, libelle);
                        }
                    } catch (LangIntegerException lie) {
                    }
                } else {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }

    }

    private void addCorpusList(Element element_xml) {
        String name = element_xml.getAttribute("classe-name");
        if (name.length() == 0) {
            errorHandler.emptyAttributeError(element_xml.getTagName(), "classe-name");
            return;
        }
        CorpusClasse corpusClasse = builder.getCorpusClasse(name);
        if (corpusClasse == null) {
            errorHandler.unknownCorpusClasseError(name);
            return;
        }
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("uri")) {
                    String uriString = XMLUtils.getData(element);
                    if (uriString.length() == 0) {
                        errorHandler.emptyElementError("uri");
                        continue;
                    }
                    try {
                        ScrutariDataURI scrutariDataURI = URIParser.parse(uriString);
                        if (scrutariDataURI instanceof CorpusURI) {
                            builder.addURI(corpusClasse, (CorpusURI) scrutariDataURI);
                        }
                    } catch (URIParseException uripe) {
                        errorHandler.uriSyntaxError(uriString);
                    }
                } else {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }
    }


    public static interface Builder {

        public CorpusClasse addClasse(String name);

        public CorpusClasse getCorpusClasse(String name);

        public void putLibelle(CorpusClasse corpusClasse, Libelle libelle);

        public boolean addURI(CorpusClasse corpusClasse, CorpusURI corpusURI);

        public void setGlobalPertinencePonderation(PertinencePonderation pertinencePonderation);

    }


    public static interface ErrorHandler {

        public void unknownTagWarning(String tagName);

        public void emptyAttributeError(String tagName, String attributeName);

        public void emptyElementError(String tagName);

        public void uriSyntaxError(String uriString);

        public void unknownCorpusClasseError(String corpusClasseName);

    }


    private static class InternalPertinencePonderation implements PertinencePonderation {

        private float rarete = 0;
        private float fraicheur = 0;
        private float position = 0;

        public float getRarete() {
            return rarete;
        }

        public float getFraicheur() {
            return fraicheur;
        }

        public float getPosition() {
            return position;
        }

        public void setRarete(float rarete) {
            this.rarete = rarete;
        }

        public void setFraicheur(float fraicheur) {
            this.fraicheur = fraicheur;
        }

        public void setPosition(float position) {
            this.position = position;
        }

    }

}
