/* SctServer - Copyright (c) 2007-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.simple;

import net.mapeadores.scrutari.data.BaseData;
import net.mapeadores.scrutari.data.CorpusData;
import net.mapeadores.scrutari.data.ThesaurusData;
import net.mapeadores.scrutari.db.CodeList;
import net.mapeadores.scrutari.db.ScrutariDB;
import net.mapeadores.scrutari.searchengine.FicheSearchResult;
import net.mapeadores.scrutari.searchengine.SearchOptions;
import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.request.RequestInfo;
import fr.exemole.sctserver.request.RequestMapUtils;
import fr.exemole.sctserver.request.WarningHandler;
import fr.exemole.sctserver.request.simple.catalog.BaseDataRequest;
import fr.exemole.sctserver.request.simple.catalog.ClasseRequest;
import fr.exemole.sctserver.request.simple.catalog.CorpusChiffreRequest;
import fr.exemole.sctserver.request.simple.catalog.CorpusDataRequest;
import fr.exemole.sctserver.request.simple.catalog.FicheResultRequest;
import fr.exemole.sctserver.request.simple.catalog.MotcleResultRequest;
import fr.exemole.sctserver.request.simple.catalog.SearchHeaderRequest;
import fr.exemole.sctserver.request.simple.catalog.ThesaurusChiffreRequest;
import fr.exemole.sctserver.request.simple.catalog.ThesaurusDataRequest;
import fr.exemole.sctserver.request.simple.catalog.TotalParBaseRequest;
import fr.exemole.sctserver.request.simple.catalog.TotalParCorpusRequest;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.SearchSequence;


/**
 *
 * @author Vincent Calame
 */
public class SimpleRequestFactory {

    private SimpleRequestFactory() {
    }

    public static SimpleRequest getSimpleRequest(SctServer sctServer, RequestMap requestMap) throws SimpleParameterErrorException {
        String param_chiffre = requestMap.getParameter("chiffre");
        if (param_chiffre != null) {
            return getChiffreRequest(sctServer.getScrutariDB(), param_chiffre, requestMap);
        }
        String param_tableau = requestMap.getParameter("tableau");
        if (param_tableau != null) {
            return getTableauRequest(sctServer, param_tableau, requestMap);
        }
        String param_data = requestMap.getParameter("data");
        if (param_data != null) {
            DataAccess dataAccess = sctServer.getScrutariDB().openDataAccess();
            SimpleRequest dataSimpleRequest = getDataRequest(sctServer.getScrutariDB(), param_data, requestMap, dataAccess);
            dataAccess.close();
            return dataSimpleRequest;
        }
        String param_idrecherche = requestMap.getParameter("idrecherche");
        if (param_idrecherche != null) {
            return getIdrechercheRequest(sctServer, param_idrecherche, requestMap);
        }
        SearchSequence searchSequence = RequestMapUtils.getSearchSequence(requestMap);
        if (searchSequence != null) {
            return getRechercheRequest(sctServer, searchSequence, requestMap);
        }
        throw new SimpleParameterErrorException();
    }

    private static SimpleRequest getRechercheRequest(SctServer sctServer, SearchSequence searchSequence, RequestMap requestMap) throws SimpleParameterErrorException {
        if (searchSequence.isEmpty()) {
            throw new SimpleParameterErrorException(SimpleParameterErrorException.EMPTY_SEARCHSEQUENCE_PARAMETER, "recherche", searchSequence.getSource());
        }
        SearchOptions searchOptions = RequestMapUtils.getSearchOptions(sctServer, requestMap, new WarningHandler());
        RequestInfo requestInfo = RequestMapUtils.getRequestInfo(requestMap);
        FicheSearchResult searchResult = sctServer.newSearch(searchSequence, searchOptions, requestInfo);
        return new SearchHeaderRequest(sctServer.getScrutariDB(), searchResult);
    }

    private static SimpleRequest getIdrechercheRequest(SctServer sctServer, String param_idrecherche, RequestMap requestMap) throws SimpleParameterErrorException {
        FicheSearchResult searchResult = sctServer.getSearchResult(param_idrecherche);
        if (searchResult == null) {
            throw new SimpleParameterErrorException(SimpleParameterErrorException.UNKNWON_IDRECHERCHE_PARAMETER, "idrecherche", param_idrecherche);
        }
        ScrutariDB scrutariDB = sctServer.getScrutariDB();
        String param_info = requestMap.getParameter("info");
        if (param_info == null) {
            throw new SimpleParameterErrorException("info");
        }
        if (param_info.equals("f")) {
            int plageStart = getPlageStart(requestMap);
            int plageLength = getPlageLength(requestMap);
            FicheResultRequest ficheResultRequest = new FicheResultRequest(scrutariDB, searchResult, plageStart, plageLength);
            ficheResultRequest.setGlobalSearchOptions(sctServer.getOptionsManager().getGlobalSearchOptions());
            ficheResultRequest.setDecalageUTF8(getDecalageUTF8(requestMap));
            return ficheResultRequest;
        } else if (param_info.equals("h")) {
            return new SearchHeaderRequest(scrutariDB, searchResult);
        } else if (param_info.equals("m")) {
            MotcleResultRequest motcleResultRequest = new MotcleResultRequest(scrutariDB, searchResult);
            motcleResultRequest.setDecalageUTF8(getDecalageUTF8(requestMap));
            motcleResultRequest.setUiLangInteger(searchResult.getLangUi());
            return motcleResultRequest;
        } else {
            throw new SimpleParameterErrorException(SimpleParameterErrorException.WRONG_PARAMETER, "info", param_info);
        }
    }

    private static SimpleRequest getDataRequest(ScrutariDB scrutariDB, String param_data, RequestMap requestMap, DataAccess dataAccess) throws SimpleParameterErrorException {
        int langUi = RequestMapUtils.getLangUi(requestMap, new WarningHandler());
        if (param_data.equals("corpus")) {
            int code = getCode(requestMap);
            CorpusData corpusData = dataAccess.getCorpusDataByCode(code);
            if (corpusData == null) {
                throw new SimpleParameterErrorException(SimpleParameterErrorException.UNKNWON_CORPUSCODE_PARAMETER, "code", requestMap.getParameter("code"));
            }
            return new CorpusDataRequest(scrutariDB, langUi, corpusData);
        } else if (param_data.equals("base")) {
            int code = getCode(requestMap);
            BaseData baseData = dataAccess.getBaseDataByCode(code);
            if (baseData == null) {
                throw new SimpleParameterErrorException(SimpleParameterErrorException.UNKNWON_BASECODE_PARAMETER, "code", requestMap.getParameter("code"));
            }
            return new BaseDataRequest(scrutariDB, langUi, baseData);
        } else if (param_data.equals("thesaurus")) {
            int code = getCode(requestMap);
            ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(code);
            if (thesaurusData == null) {
                throw new SimpleParameterErrorException(SimpleParameterErrorException.UNKNWON_THESAURUSCODE_PARAMETER, "code", requestMap.getParameter("code"));
            }
            return new ThesaurusDataRequest(scrutariDB, langUi, thesaurusData);
        }
        throw new SimpleParameterErrorException(SimpleParameterErrorException.WRONG_PARAMETER, "info", param_data);
    }

    private static SimpleRequest getTableauRequest(SctServer sctServer, String param_tableau, RequestMap requestMap) throws SimpleParameterErrorException {
        ScrutariDB scrutariDB = sctServer.getScrutariDB();
        int langUi = RequestMapUtils.getLangUi(requestMap, new WarningHandler());
        if (param_tableau.equals("corpus")) {
            return new CorpusDataRequest(scrutariDB, langUi);
        } else if (param_tableau.equals("thesaurus")) {
            return new ThesaurusDataRequest(scrutariDB, langUi);
        } else if (param_tableau.equals("base")) {
            return new BaseDataRequest(scrutariDB, langUi);
        } else if (param_tableau.equals("classe")) {
            return new ClasseRequest(sctServer, langUi);
        }
        throw new SimpleParameterErrorException(SimpleParameterErrorException.WRONG_PARAMETER, "tableau", param_tableau);
    }

    private static SimpleRequest getChiffreRequest(ScrutariDB scrutariDB, String chiffreParam, RequestMap requestMap) throws SimpleParameterErrorException {
        if (chiffreParam.equals("codecorpus")) {
            int codebase = getCodeBase(requestMap);
            String param_nomcorpus = requestMap.getParameter("nomcorpus");
            if (param_nomcorpus == null) {
                throw new SimpleParameterErrorException("nomcorpus");
            }
            return new CorpusChiffreRequest(scrutariDB, codebase, param_nomcorpus);
        } else if (chiffreParam.equals("codethesaurus")) {
            int codebase = getCodeBase(requestMap);
            String param_nomthesaurus = requestMap.getParameter("nomthesaurus");
            if (param_nomthesaurus == null) {
                throw new SimpleParameterErrorException("nomthesaurus");
            }
            return new ThesaurusChiffreRequest(scrutariDB, codebase, param_nomthesaurus);
        } else if (chiffreParam.equals("total")) {
            int[] langIntegerArray = RequestMapUtils.getLangIntegerArray(requestMap);
            DataAccess dataAccess = scrutariDB.openDataAccess();
            WarningHandler warningHandler = new WarningHandler();
            CodeList codecorpusList = RequestMapUtils.getCorpusCodeList(dataAccess, requestMap, warningHandler);
            if (codecorpusList != null) {
                return new TotalParCorpusRequest(scrutariDB, codecorpusList, langIntegerArray);
            }
            CodeList codebaseList = RequestMapUtils.getBaseCodeList(dataAccess, requestMap, warningHandler);
            dataAccess.close();
            return new TotalParBaseRequest(scrutariDB, codebaseList, langIntegerArray);
        }
        throw new SimpleParameterErrorException(SimpleParameterErrorException.WRONG_PARAMETER, "chiffre", chiffreParam);
    }

    private static int getCodeBase(RequestMap requestMap) throws SimpleParameterErrorException {
        String param_codebase = requestMap.getParameter("codebase");
        if (param_codebase == null) {
            throw new SimpleParameterErrorException("codebase");
        }
        try {
            int codebase = Integer.parseInt(param_codebase);
            return codebase;
        } catch (NumberFormatException nfe) {
            throw new SimpleParameterErrorException(SimpleParameterErrorException.NOTINTEGER_PARAMETER, "codebase", param_codebase);
        }

    }

    private static int getCode(RequestMap requestMap) throws SimpleParameterErrorException {
        String param_code = requestMap.getParameter("code");
        if (param_code == null) {
            throw new SimpleParameterErrorException("code");
        }
        try {
            int code = Integer.parseInt(param_code);
            return code;
        } catch (NumberFormatException nfe) {
            throw new SimpleParameterErrorException(SimpleParameterErrorException.NOTINTEGER_PARAMETER, "code", param_code);
        }
    }

    private static boolean getDecalageUTF8(RequestMap requestMap) {
        String param_dclg = requestMap.getParameter("dclg");
        boolean decalageutf8 = false;
        if ((param_dclg != null) && (param_dclg.equals("1"))) {
            decalageutf8 = true;
        }
        return decalageutf8;
    }

    private static int getPlageStart(RequestMap requestMap) {
        String param_debutplage = requestMap.getParameter("dbtplage");
        if (param_debutplage == null) {
            return 0;
        }
        try {
            return Integer.parseInt(param_debutplage);
        } catch (NumberFormatException nfe) {
            return 0;
        }
    }

    private static int getPlageLength(RequestMap requestMap) {
        String param_debutplage = requestMap.getParameter("lngplage");
        if (param_debutplage == null) {
            return -1;
        }
        try {
            return Integer.parseInt(param_debutplage);
        } catch (NumberFormatException nfe) {
            return -1;
        }
    }

}
