/* SctServer - Copyright (c) 2005-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.xml;

import net.scrutari.datauri.FicheURI;
import net.mapeadores.scrutari.data.FicheData;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.util.DataUtils;
import net.mapeadores.scrutari.lexie.FieldNumbers;
import net.mapeadores.scrutari.searchengine.FicheSearchResult;
import net.mapeadores.scrutari.searchengine.FicheSearchResultInfo;
import net.mapeadores.scrutari.searchengine.SearchTokenPositionList;
import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.config.ConfigException;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.xml.XMLUtils;
import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.scrutari.db.ScrutariDB;
import net.mapeadores.scrutari.searchengine.FicheSearchResultGroup;


/**
 * Classe produisant l'arbre XML des fiches.
 *
 * @author Vincent Calame
 */
public class FichesRequest implements FieldNumbers {

    private FicheSearchResult searchResult;
    private ScrutariDB scrutariDB;
    private int langUi;

    /**
     * Construit une nouvelle instance de FichesRequest à partir de la recherche d'identifiant searchId.
     */
    public FichesRequest(String searchid, SctServer sctServer) throws XmlRequestException {
        if (searchid == null) {
            throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE, XmlRequestException.MISSING_SEARCHID_PARAMETER);
        }
        try {
            searchResult = sctServer.getSearchResult(searchid);
        } catch (ConfigException ce) {
            if (ce.hasSource()) {
                throw new XmlRequestException(ce.getSource());
            } else {
                throw new XmlRequestException(ce);
            }
        }
        if (searchResult == null) {
            throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE, XmlRequestException.WRONG_SEARCHID_PARAMETER, searchid);
        }
        scrutariDB = sctServer.getScrutariDB();
        langUi = searchResult.getLangUi();
    }

    /**
     * Écrit l'arbre XML des fiches comme réponse à une requête de servlet.
     */
    public void printResponse(HttpServletResponse reponse) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        int tabulations = 0;
        reponse.setContentType("text/xml;charset=UTF-8");
        PrintWriter pw = reponse.getWriter();
        pw.print("<?xml version='1.0' encoding='UTF-8' standalone='no' ?>");
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<!DOCTYPE fiches SYSTEM \"dtd/fiches.dtd\">");
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<fiches id=\"fiches\" search-id=\"");
        pw.print(searchResult.getIdentifiant());
        pw.print("\" fiche-count=\"");
        pw.print(String.valueOf(searchResult.getFicheCount()));
        pw.print("\"");
        pw.print(">");
        int groupCount = searchResult.getFicheSearchResultGroupCount();
        for (int i = 0; i < groupCount; i++) {
            FicheSearchResultGroup ficheSearchResultGroup = searchResult.getFicheSearchResultGroup(i);
            int infoCount = ficheSearchResultGroup.getFicheSearchResultCount();
            for (int j = 0; j < infoCount; j++) {
                printFicheResult(pw, ficheSearchResultGroup.getFicheSearchResultInfo(j), tabulations + 1, dataAccess);
            }
        }
        XMLUtils.printTabs(pw, tabulations);
        pw.print("</fiches>");
        dataAccess.close();
    }

    /*
     * Écrit les informations liées à une fiche d'un résultat. Méthode appelée par printResponse.
     */
    private void printFicheResult(PrintWriter pw, FicheSearchResultInfo ficheResult, int tabulations, DataAccess dataAccess) {
        int code = ficheResult.getCode();
        String code_string = String.valueOf(code);
        FicheData ficheData = dataAccess.getFicheDataByCode(code);
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<fiche id=\"");
        pw.print(code_string);
        pw.print("\" fiche-code=\"");
        pw.print(code_string);
        pw.print("\" corpus-code=\"");
        pw.print(String.valueOf(ficheData.getCodeCorpus()));
        pw.print("\" lang=\"");
        pw.print(XmlRequestUtils.getLangString(ficheData.getLangInteger()));
        int dateInteger = ficheData.getDateInteger();
        if (dateInteger != DateInteger.NULL) {
            pw.print("\" date=\"");
            pw.print(DateInteger.toString(dateInteger));
        }
        String href = DataUtils.getHref(ficheData, dataAccess, langUi);
        if (href != null) {
            pw.print("\" href=\"");
            XMLUtils.printEscape(pw, href);
        }
        pw.print("\"");
        pw.print(">");
        String titre = ficheData.getTitre();
        if (titre != null) {
            SearchTokenPositionList positionList = ficheResult.getSubstringPositionList(TITRE_FIELDNUMBER);
            XMLUtils.printTabs(pw, tabulations + 1);
            pw.print("<titre id=\"");
            pw.print(code_string);
            pw.print("_titre\" role=\"");
            if (positionList == null) {
                pw.print("0");
            } else {
                pw.print("1");
            }
            pw.print("\">");
            XmlRequestUtils.printPositionList(pw, titre, positionList);
            pw.print("</titre>");
        }
        String soustitre = ficheData.getSoustitre();
        if (soustitre != null) {
            SearchTokenPositionList positionList = ficheResult.getSubstringPositionList(SOUSTITRE_FIELDNUMBER);
            XMLUtils.printTabs(pw, tabulations + 1);
            pw.print("<soustitre id=\"");
            pw.print(code_string);
            pw.print("_soustitre\" role=\"");
            if (positionList == null) {
                pw.print("0");
            } else {
                pw.print("1");
            }
            pw.print("\">");
            XmlRequestUtils.printPositionList(pw, soustitre, positionList);
            pw.print("</soustitre>");
        }
        pw.print("<numero id=\"");
        pw.print(code_string);
        pw.print("_numero\">");
        FicheURI ficheURI = ficheData.getFicheURI();
        pw.print(ficheURI.getFicheId());
        pw.print("</numero>");
        for (int p = 1; p <= ficheData.getComplementMaxNumber(); p++) {
            String complement = ficheData.getComplementByNumber(p);
            if (complement != null) {
                SearchTokenPositionList positionList = ficheResult.getSubstringPositionList(p);
                XMLUtils.printTabs(pw, tabulations + 1);
                pw.print("<complement id=\"");
                pw.print(code_string);
                pw.print("_complement_");
                pw.print(String.valueOf(p));
                pw.print("\" complement-number=\"");
                pw.print(String.valueOf(p));
                pw.print("\" role=\"");
                if (positionList == null) {
                    pw.print("0");
                } else {
                    pw.print("1");
                }
                pw.print("\">");
                XmlRequestUtils.printPositionList(pw, complement, positionList);
                pw.print("</complement>");
            }
        }
        for (int i = 0; i < ficheResult.getMotcleCount(); i++) {
            int codeMotcle = ficheResult.getMotcleCode(i);
            XMLUtils.printTabs(pw, tabulations + 1);
            pw.print("<mtcl motcle-code=\"");
            pw.print(String.valueOf(codeMotcle));
            pw.print("\"/>");
        }
        XMLUtils.printTabs(pw, tabulations);
        pw.print("</fiche>");
    }

}
