/*  SctServer_Directory - Copyright (c) 2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.directory.cache;

import fr.exemole.sctserver.storage.SctStorageException;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.scrutari.db.LexieCacheReader;
import net.mapeadores.scrutari.db.LexieCacheReaderFactory;
import net.mapeadores.scrutari.db.LexieCacheWriter;
import net.mapeadores.scrutari.lexie.FicheLexification;
import net.mapeadores.scrutari.lexie.LexieOccurrences;
import net.mapeadores.scrutari.lexie.LexieOccurrencesList;
import net.mapeadores.scrutari.lexie.LexieUtils;
import net.mapeadores.scrutari.lexie.MotcleLexification;
import net.mapeadores.scrutari.lexie.ScrutariLexieUnit;
import net.mapeadores.scrutari.lexie.ScrutariLexieUnitList;
import net.mapeadores.util.primitives.io.DataInputPrimitivesReader;
import net.mapeadores.util.primitives.io.DataOutputPrimitivesWriter;


/**
 *
 * @author Vincent Calame
 */
public class DirectoryLexieCacheWriter implements LexieCacheWriter {

    private RandomAccessFile occurrencesRandomAccessFile;
    private DataOutputPrimitivesWriter occurrencesPrimitivesWriter;
    private File occurrencesFile;
    private RandomAccessFile lexificationRandomAccessFile;
    private DataOutputPrimitivesWriter lexificationPrimitivesWriter;
    private File lexificationFile;
    private RandomAccessFile unitRandomAccessFile;
    private DataOutputPrimitivesWriter unitPrimitivesWriter;
    private File unitFile;
    private Map<Integer, Long> lexificationPositionMap = new HashMap<Integer, Long>();
    private Map<OccurrencesKey, Long> occurrencesPositionMap = new HashMap<OccurrencesKey, Long>();
    private Map<Integer, Long> unitPositionMap = new HashMap<Integer, Long>();

    public DirectoryLexieCacheWriter(File cacheDirectory) {
        File occurrences = new File(cacheDirectory, "lexie-occurrences.primitives");
        if (occurrences.exists()) {
            occurrences.delete();
        }
        try {
            this.occurrencesRandomAccessFile = new RandomAccessFile(occurrences, "rw");
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
        this.occurrencesPrimitivesWriter = new DataOutputPrimitivesWriter(occurrencesRandomAccessFile);
        this.occurrencesFile = occurrences;
        File lexification = new File(cacheDirectory, "lexie-lexification.primitives");
        if (lexification.exists()) {
            lexification.delete();
        }
        try {
            this.lexificationRandomAccessFile = new RandomAccessFile(lexification, "rw");
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
        this.lexificationPrimitivesWriter = new DataOutputPrimitivesWriter(lexificationRandomAccessFile);
        this.lexificationFile = lexification;
        File unit = new File(cacheDirectory, "lexie-unit.primitives");
        if (unit.exists()) {
            unit.delete();
        }
        try {
            this.unitRandomAccessFile = new RandomAccessFile(unit, "rw");
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
        this.unitPrimitivesWriter = new DataOutputPrimitivesWriter(unitRandomAccessFile);
        this.unitFile = unit;
    }

    public void cacheScrutariLexieUnitList(int langInteger, ScrutariLexieUnitList scrutariUnitList) {
        try {
            long position = unitRandomAccessFile.getFilePointer();
            unitPositionMap.put(langInteger, position);
            int count = scrutariUnitList.getScrutariLexieUnitCount();
            unitPrimitivesWriter.writeInt(count);
            for (int i = 0; i < count; i++) {
                ScrutariLexieUnit.toPrimitives(scrutariUnitList.getScrutariLexieUnit(i), unitPrimitivesWriter);
            }
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    public void cacheLexieOccurrencesList(long lexieId, int subsetCode, LexieOccurrencesList lexieOccurencesList) {
        try {
            long position = occurrencesRandomAccessFile.getFilePointer();
            occurrencesPositionMap.put(new OccurrencesKey(lexieId, subsetCode), position);
            int count = lexieOccurencesList.getLexieOccurrencesCount();
            occurrencesPrimitivesWriter.writeInt(count);
            for (int i = 0; i < count; i++) {
                LexieOccurrences.toPrimitives(lexieOccurencesList.getLexieOccurrences(i), occurrencesPrimitivesWriter);
            }
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    public void cacheFicheLexification(int codefiche, FicheLexification ficheLexification) {
        try {
            long position = lexificationRandomAccessFile.getFilePointer();
            lexificationPositionMap.put(codefiche, position);
            FicheLexification.toPrimitives(ficheLexification, lexificationPrimitivesWriter);
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    public void cacheMotcleLexification(int codemotcle, MotcleLexification motcleLexification) {
        try {
            long position = lexificationRandomAccessFile.getFilePointer();
            lexificationPositionMap.put(codemotcle, position);
            MotcleLexification.toPrimitives(motcleLexification, lexificationPrimitivesWriter);
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    public LexieCacheReaderFactory endCacheWrite() {
        try {
            occurrencesPrimitivesWriter = null;
            occurrencesRandomAccessFile.close();
            occurrencesRandomAccessFile = null;
            lexificationPrimitivesWriter = null;
            lexificationRandomAccessFile.close();
            lexificationRandomAccessFile = null;
            unitPrimitivesWriter = null;
            unitRandomAccessFile.close();
            unitRandomAccessFile = null;
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
        return new InternalLexieCacheReaderFactory(occurrencesFile, lexificationFile, unitFile, lexificationPositionMap, occurrencesPositionMap, unitPositionMap);
    }


    private static class InternalLexieCacheReaderFactory implements LexieCacheReaderFactory {

        private File occurrencesFile;
        private File lexificationFile;
        private File unitFile;
        private Map<Integer, Long> lexificationPositionMap;
        private Map<OccurrencesKey, Long> occurrencesPositionMap;
        private Map<Integer, Long> unitPositionMap = new HashMap<Integer, Long>();

        private InternalLexieCacheReaderFactory(File occurrencesFile, File lexificationFile, File unitFile, Map<Integer, Long> lexificationPositionMap, Map<OccurrencesKey, Long> occurrencesPositionMap, Map<Integer, Long> unitPositionMap) {
            this.occurrencesFile = occurrencesFile;
            this.lexificationFile = lexificationFile;
            this.unitFile = unitFile;
            this.lexificationPositionMap = lexificationPositionMap;
            this.occurrencesPositionMap = occurrencesPositionMap;
            this.unitPositionMap = unitPositionMap;
        }

        public LexieCacheReader newReader() {
            return new InternalLexieCacheReader();
        }


        private class InternalLexieCacheReader implements LexieCacheReader {

            private RandomAccessFile occurrencesRandomAccessFile;
            private DataInputPrimitivesReader occurrencesPrimitivesReader;
            private RandomAccessFile lexificationRandomAccessFile;
            private DataInputPrimitivesReader lexificationPrimitivesReader;
            private RandomAccessFile unitRandomAccessFile;
            private DataInputPrimitivesReader unitPrimitivesReader;

            private InternalLexieCacheReader() {
                try {
                    this.unitRandomAccessFile = new RandomAccessFile(unitFile, "r");
                } catch (IOException ioe) {
                    throw new SctStorageException(ioe);
                }
                this.unitPrimitivesReader = new DataInputPrimitivesReader(unitRandomAccessFile);
            }

            public ScrutariLexieUnitList getScrutariLexieUnitList(int langInteger) {
                Long position = unitPositionMap.get(langInteger);
                if (position == null) {
                    return LexieUtils.EMPTY_SCRUTARILEXIEUNITLIST;
                }
                try {
                    if (unitRandomAccessFile == null) {
                        unitRandomAccessFile = new RandomAccessFile(unitFile, "r");
                        unitPrimitivesReader = new DataInputPrimitivesReader(unitRandomAccessFile);
                    }
                    unitRandomAccessFile.seek(position.longValue());
                    int length = unitRandomAccessFile.readInt();
                    if (length == 0) {
                        return LexieUtils.EMPTY_SCRUTARILEXIEUNITLIST;
                    } else if (length == 1) {
                        return LexieUtils.toScrutariLexieUnitList(ScrutariLexieUnit.fromPrimitives(unitPrimitivesReader));
                    } else {
                        ScrutariLexieUnit[] lexieOccurrencesArray = new ScrutariLexieUnit[length];
                        for (int i = 0; i < length; i++) {
                            lexieOccurrencesArray[i] = ScrutariLexieUnit.fromPrimitives(unitPrimitivesReader);
                        }
                        return LexieUtils.toScrutariLexieUnitList(lexieOccurrencesArray);
                    }
                } catch (IOException ioe) {
                    throw new SctStorageException(ioe);
                }
            }

            public LexieOccurrencesList getLexieOccurrencesList(long lexieId, int subsetCode) {
                Long position = occurrencesPositionMap.get(new OccurrencesKey(lexieId, subsetCode));
                if (position == null) {
                    return LexieUtils.EMPTY_LEXIEOCCURRENCESLIST;
                }
                try {
                    if (occurrencesRandomAccessFile == null) {
                        occurrencesRandomAccessFile = new RandomAccessFile(occurrencesFile, "r");
                        occurrencesPrimitivesReader = new DataInputPrimitivesReader(occurrencesRandomAccessFile);
                    }
                    occurrencesRandomAccessFile.seek(position.longValue());
                    int length = occurrencesRandomAccessFile.readInt();
                    if (length == 0) {
                        return LexieUtils.EMPTY_LEXIEOCCURRENCESLIST;
                    } else if (length == 1) {
                        return LexieUtils.toLexieOccurrencesList(LexieOccurrences.fromPrimitives(occurrencesPrimitivesReader));
                    } else {
                        LexieOccurrences[] lexieOccurrencesArray = new LexieOccurrences[length];
                        for (int i = 0; i < length; i++) {
                            lexieOccurrencesArray[i] = LexieOccurrences.fromPrimitives(occurrencesPrimitivesReader);
                        }
                        return LexieUtils.toLexieOccurrencesList(lexieOccurrencesArray);
                    }
                } catch (IOException ioe) {
                    throw new SctStorageException(ioe);
                }
            }

            public FicheLexification getFicheLexification(int codefiche) {
                try {
                    boolean done = testLexification(codefiche);
                    if (!done) {
                        return null;
                    }
                    return FicheLexification.fromPrimitives(lexificationPrimitivesReader);
                } catch (IOException ioe) {
                    throw new SctStorageException(ioe);
                }
            }

            public MotcleLexification getMotcleLexification(int codemotcle) {
                try {
                    boolean done = testLexification(codemotcle);
                    if (!done) {
                        return null;
                    }
                    return MotcleLexification.fromPrimitives(lexificationPrimitivesReader);
                } catch (IOException ioe) {
                    throw new SctStorageException(ioe);
                }
            }

            public void close() {
                try {
                    if (occurrencesRandomAccessFile != null) {
                        occurrencesPrimitivesReader = null;
                        occurrencesRandomAccessFile.close();
                        occurrencesRandomAccessFile = null;
                    }
                    if (lexificationRandomAccessFile != null) {
                        lexificationPrimitivesReader = null;
                        lexificationRandomAccessFile.close();
                        lexificationRandomAccessFile = null;
                    }
                    if (unitRandomAccessFile != null) {
                        unitPrimitivesReader = null;
                        unitRandomAccessFile.close();
                        unitRandomAccessFile = null;
                    }
                } catch (IOException ioe) {
                    throw new SctStorageException(ioe);
                }
            }

            private boolean testLexification(int elementCode) throws IOException {
                Long position = lexificationPositionMap.get(elementCode);
                if (position == null) {
                    return false;
                }
                if (lexificationRandomAccessFile == null) {
                    lexificationRandomAccessFile = new RandomAccessFile(lexificationFile, "r");
                    lexificationPrimitivesReader = new DataInputPrimitivesReader(lexificationRandomAccessFile);
                }
                lexificationRandomAccessFile.seek(position.longValue());
                return true;
            }

        }

    }


    private static final class OccurrencesKey {

        private long lexieId;
        private int subsetCode;

        private OccurrencesKey(long lexieId, int subsetCode) {
            this.lexieId = lexieId;
            this.subsetCode = subsetCode;
        }

        @Override
        public boolean equals(Object other) {
            if (other == null) {
                return false;
            }
            if (other == this) {
                return true;
            }
            if (other.getClass() != this.getClass()) {
                return false;
            }
            OccurrencesKey otherOccurrencesKey = (OccurrencesKey) other;
            return (otherOccurrencesKey.lexieId == this.lexieId) && (otherOccurrencesKey.subsetCode == this.subsetCode);
        }

        @Override
        public int hashCode() {
            return ((int) lexieId) + subsetCode;
        }

    }

}
